"""
    :codeauthor: Rupesh Tare <rupesht@saltstack.com>
"""


import salt.modules.pw_user as pw_user
from salt.exceptions import CommandExecutionError
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase, skipIf

try:
    import pwd

    HAS_PWD = True
except ImportError:
    HAS_PWD = False


@skipIf(not HAS_PWD, "These tests can only run on systems with the python pwd module")
class PwUserTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.modules.pw_user
    """

    def setup_loader_modules(self):
        return {pw_user: {}}

    def test_add(self):
        """
        Test for adding a user
        """
        with patch.dict(pw_user.__grains__, {"os_family": "RedHat"}):
            mock = MagicMock(return_value=0)
            with patch.dict(pw_user.__salt__, {"cmd.retcode": mock}):
                self.assertTrue(pw_user.add("a"))

    def test_delete(self):
        """
        Test for deleting a user
        """
        mock = MagicMock(return_value=0)
        with patch.dict(pw_user.__salt__, {"cmd.retcode": mock}):
            self.assertTrue(pw_user.delete("A"))

    def test_getent(self):
        """
        Test if user.getent already have a value
        """
        mock_user = "saltdude"

        class MockData:
            pw_name = mock_user

        with patch("pwd.getpwall", MagicMock(return_value=[MockData()])):
            with patch.dict(pw_user.__context__, {"user.getent": mock_user}):
                self.assertEqual(pw_user.getent(), mock_user)

                with patch.object(pw_user, "info", MagicMock(return_value=mock_user)):
                    self.assertEqual(pw_user.getent(True)[0], mock_user)

    def test_chuid(self):
        """
        Test if user id given is same as previous id
        """
        mock = MagicMock(return_value={"uid": "A"})
        with patch.object(pw_user, "info", mock):
            self.assertTrue(pw_user.chuid("name", "A"))

        mock = MagicMock(return_value=None)
        with patch.dict(pw_user.__salt__, {"cmd.run": mock}):
            mock = MagicMock(side_effect=[{"uid": "A"}, {"uid": "A"}])
            with patch.object(pw_user, "info", mock):
                self.assertFalse(pw_user.chuid("name", "B"))

        mock = MagicMock(return_value=None)
        with patch.dict(pw_user.__salt__, {"cmd.run": mock}):
            mock = MagicMock(side_effect=[{"uid": "A"}, {"uid": "B"}])
            with patch.object(pw_user, "info", mock):
                self.assertTrue(pw_user.chuid("name", "A"))

    def test_chgid(self):
        """
        Test if group id given is same as previous id
        """
        mock = MagicMock(return_value={"gid": 1})
        with patch.object(pw_user, "info", mock):
            self.assertTrue(pw_user.chgid("name", 1))

        mock = MagicMock(return_value=None)
        with patch.dict(pw_user.__salt__, {"cmd.run": mock}):
            mock = MagicMock(side_effect=[{"gid": 2}, {"gid": 2}])
            with patch.object(pw_user, "info", mock):
                self.assertFalse(pw_user.chgid("name", 1))

        mock = MagicMock(return_value=None)
        with patch.dict(pw_user.__salt__, {"cmd.run": mock}):
            mock = MagicMock(side_effect=[{"gid": 1}, {"gid": 2}])
            with patch.object(pw_user, "info", mock):
                self.assertTrue(pw_user.chgid("name", 1))

    def test_chshell(self):
        """
        Test if shell given is same as previous shell
        """
        mock = MagicMock(return_value={"shell": "A"})
        with patch.object(pw_user, "info", mock):
            self.assertTrue(pw_user.chshell("name", "A"))

        mock = MagicMock(return_value=None)
        with patch.dict(pw_user.__salt__, {"cmd.run": mock}):
            mock = MagicMock(side_effect=[{"shell": "B"}, {"shell": "B"}])
            with patch.object(pw_user, "info", mock):
                self.assertFalse(pw_user.chshell("name", "A"))

        mock = MagicMock(return_value=None)
        with patch.dict(pw_user.__salt__, {"cmd.run": mock}):
            mock = MagicMock(side_effect=[{"shell": "A"}, {"shell": "B"}])
            with patch.object(pw_user, "info", mock):
                self.assertTrue(pw_user.chshell("name", "A"))

    def test_chhome(self):
        """
        Test if home directory given is same as previous home directory
        """
        mock = MagicMock(return_value={"home": "A"})
        with patch.object(pw_user, "info", mock):
            self.assertTrue(pw_user.chhome("name", "A"))

        mock = MagicMock(return_value=None)
        with patch.dict(pw_user.__salt__, {"cmd.run": mock}):
            mock = MagicMock(side_effect=[{"home": "B"}, {"home": "B"}])
            with patch.object(pw_user, "info", mock):
                self.assertFalse(pw_user.chhome("name", "A"))

        mock = MagicMock(return_value=None)
        with patch.dict(pw_user.__salt__, {"cmd.run": mock}):
            mock = MagicMock(side_effect=[{"home": "A"}, {"home": "B"}])
            with patch.object(pw_user, "info", mock):
                self.assertTrue(pw_user.chhome("name", "A"))

    def test_chgroups(self):
        """
        Test if no group needs to be added
        """
        mock = MagicMock(return_value=False)
        with patch.dict(pw_user.__salt__, {"cmd.retcode": mock}):
            mock = MagicMock(return_value=["a", "b", "c", "d"])
            with patch.object(pw_user, "list_groups", mock):
                self.assertTrue(pw_user.chgroups("name", "a, b, c, d"))

        mock = MagicMock(return_value=False)
        with patch.dict(pw_user.__salt__, {"cmd.retcode": mock}):
            mock = MagicMock(return_value=["a", "b"])
            with patch.object(pw_user, "list_groups", mock):
                self.assertTrue(pw_user.chgroups("name", "a, b, c"))

    def test_chfullname(self):
        """
        Change the user's Full Name
        """
        mock = MagicMock(return_value=False)
        with patch.object(pw_user, "_get_gecos", mock):
            self.assertFalse(pw_user.chfullname("name", "fullname"))

        mock = MagicMock(return_value={"fullname": "fullname"})
        with patch.object(pw_user, "_get_gecos", mock):
            self.assertTrue(pw_user.chfullname("name", "fullname"))

        mock = MagicMock(return_value={"fullname": "Unicøde name ①③②"})
        with patch.object(pw_user, "_get_gecos", mock):
            self.assertTrue(pw_user.chfullname("name", "Unicøde name ①③②"))

        mock = MagicMock(return_value={"fullname": "fullname"})
        with patch.object(pw_user, "_get_gecos", mock):
            mock = MagicMock(return_value=None)
            with patch.dict(pw_user.__salt__, {"cmd.run": mock}):
                mock = MagicMock(return_value={"fullname": "fullname2"})
                with patch.object(pw_user, "info", mock):
                    self.assertFalse(pw_user.chfullname("name", "fullname1"))

        mock = MagicMock(return_value={"fullname": "fullname2"})
        with patch.object(pw_user, "_get_gecos", mock):
            mock = MagicMock(return_value=None)
            with patch.dict(pw_user.__salt__, {"cmd.run": mock}):
                mock = MagicMock(return_value={"fullname": "fullname2"})
                with patch.object(pw_user, "info", mock):
                    self.assertFalse(pw_user.chfullname("name", "fullname1"))

    def test_chroomnumber(self):
        """
        Change the user's Room Number
        """
        mock = MagicMock(return_value=False)
        with patch.object(pw_user, "_get_gecos", mock):
            self.assertFalse(pw_user.chroomnumber("name", 1))

        mock = MagicMock(return_value={"roomnumber": "Unicøde room ①③②"})
        with patch.object(pw_user, "_get_gecos", mock):
            self.assertTrue(pw_user.chroomnumber("name", "Unicøde room ①③②"))

        mock = MagicMock(return_value={"roomnumber": "1"})
        with patch.object(pw_user, "_get_gecos", mock):
            self.assertTrue(pw_user.chroomnumber("name", 1))

        mock = MagicMock(return_value={"roomnumber": "2"})
        with patch.object(pw_user, "_get_gecos", mock):
            mock = MagicMock(return_value=None)
            with patch.dict(pw_user.__salt__, {"cmd.run": mock}):
                mock = MagicMock(return_value={"roomnumber": "3"})
                with patch.object(pw_user, "info", mock):
                    self.assertFalse(pw_user.chroomnumber("name", 1))

        mock = MagicMock(return_value={"roomnumber": "3"})
        with patch.object(pw_user, "_get_gecos", mock):
            mock = MagicMock(return_value=None)
            with patch.dict(pw_user.__salt__, {"cmd.run": mock}):
                mock = MagicMock(return_value={"roomnumber": "3"})
                with patch.object(pw_user, "info", mock):
                    self.assertFalse(pw_user.chroomnumber("name", 1))

    def test_chworkphone(self):
        """
        Change the user's Work Phone
        """
        mock = MagicMock(return_value=False)
        with patch.object(pw_user, "_get_gecos", mock):
            self.assertFalse(pw_user.chworkphone("name", 1))

        mock = MagicMock(return_value={"workphone": "1"})
        with patch.object(pw_user, "_get_gecos", mock):
            self.assertTrue(pw_user.chworkphone("name", 1))

        mock = MagicMock(return_value={"workphone": "Unicøde phone number ①③②"})
        with patch.object(pw_user, "_get_gecos", mock):
            self.assertTrue(pw_user.chworkphone("name", "Unicøde phone number ①③②"))

        mock = MagicMock(return_value={"workphone": "2"})
        with patch.object(pw_user, "_get_gecos", mock):
            mock = MagicMock(return_value=None)
            with patch.dict(pw_user.__salt__, {"cmd.run": mock}):
                mock = MagicMock(return_value={"workphone": "3"})
                with patch.object(pw_user, "info", mock):
                    self.assertFalse(pw_user.chworkphone("name", 1))

        mock = MagicMock(return_value={"workphone": "3"})
        with patch.object(pw_user, "_get_gecos", mock):
            mock = MagicMock(return_value=None)
            with patch.dict(pw_user.__salt__, {"cmd.run": mock}):
                mock = MagicMock(return_value={"workphone": "3"})
                with patch.object(pw_user, "info", mock):
                    self.assertFalse(pw_user.chworkphone("name", 1))

    def test_chhomephone(self):
        """
        Change the user's Home Phone
        """
        mock = MagicMock(return_value=False)
        with patch.object(pw_user, "_get_gecos", mock):
            self.assertFalse(pw_user.chhomephone("name", 1))

        mock = MagicMock(return_value={"homephone": "1"})
        with patch.object(pw_user, "_get_gecos", mock):
            self.assertTrue(pw_user.chhomephone("name", 1))

        mock = MagicMock(return_value={"homephone": "Unicøde phone number ①③②"})
        with patch.object(pw_user, "_get_gecos", mock):
            self.assertTrue(pw_user.chhomephone("name", "Unicøde phone number ①③②"))

        mock = MagicMock(return_value={"homephone": "2"})
        with patch.object(pw_user, "_get_gecos", mock):
            mock = MagicMock(return_value=None)
            with patch.dict(pw_user.__salt__, {"cmd.run": mock}):
                mock = MagicMock(return_value={"homephone": "3"})
                with patch.object(pw_user, "info", mock):
                    self.assertFalse(pw_user.chhomephone("name", 1))

        mock = MagicMock(return_value={"homephone": "3"})
        with patch.object(pw_user, "_get_gecos", mock):
            mock = MagicMock(return_value=None)
            with patch.dict(pw_user.__salt__, {"cmd.run": mock}):
                mock = MagicMock(return_value={"homephone": "3"})
                with patch.object(pw_user, "info", mock):
                    self.assertFalse(pw_user.chhomephone("name", 1))

    def test_info(self):
        """
        Return user information
        """
        self.assertEqual(pw_user.info("name"), {})

        mock = MagicMock(
            return_value=pwd.struct_passwd(
                (
                    "_TEST_GROUP",
                    "*",
                    83,
                    83,
                    "AMaViS Daemon",
                    "/var/virusmails",
                    "/usr/bin/false",
                )
            )
        )
        with patch.object(pwd, "getpwnam", mock):
            mock = MagicMock(return_value="Group Name")
            with patch.object(pw_user, "list_groups", mock):
                self.assertEqual(pw_user.info("name")["name"], "_TEST_GROUP")

    def test_list_groups(self):
        """
        Return a list of groups the named user belongs to
        """
        mock_group = "saltgroup"

        with patch(
            "salt.utils.user.get_group_list", MagicMock(return_value=[mock_group])
        ):
            self.assertEqual(pw_user.list_groups("name"), [mock_group])

    def test_list_users(self):
        """
        Return a list of all users
        """
        mock_user = "saltdude"

        class MockData:
            pw_name = mock_user

        with patch("pwd.getpwall", MagicMock(return_value=[MockData()])):
            self.assertEqual(pw_user.list_users(), [mock_user])

    def test_rename(self):
        """
        Change the username for a named user
        """
        mock = MagicMock(return_value=False)
        with patch.object(pw_user, "info", mock):
            self.assertRaises(CommandExecutionError, pw_user.rename, "name", 1)

        mock = MagicMock(return_value=True)
        with patch.object(pw_user, "info", mock):
            self.assertRaises(CommandExecutionError, pw_user.rename, "name", 1)

        mock = MagicMock(return_value=None)
        with patch.dict(pw_user.__salt__, {"cmd.run": mock}):
            mock = MagicMock(side_effect=[{"name": ""}, False, {"name": "name"}])
            with patch.object(pw_user, "info", mock):
                self.assertTrue(pw_user.rename("name", "name"))

        mock = MagicMock(return_value=None)
        with patch.dict(pw_user.__salt__, {"cmd.run": mock}):
            mock = MagicMock(side_effect=[{"name": ""}, False, {"name": ""}])
            with patch.object(pw_user, "info", mock):
                self.assertFalse(pw_user.rename("name", "name"))



"""
Tests for salt.modules.zfs

:codeauthor:    Nitin Madhok <nmadhok@g.clemson.edu>, Jorge Schrauwen <sjorge@blackdot.be>
:maintainer:    Jorge Schrauwen <sjorge@blackdot.be>
:maturity:      new
:depends:       salt.utils.zfs
:platform:      illumos,freebsd,linux
"""

import pytest
import salt.loader
import salt.modules.zfs as zfs
import salt.utils.zfs
from salt.utils.dateutils import strftime
from salt.utils.odict import OrderedDict
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase
from tests.support.zfs import ZFSMockData


class ZfsTestCase(TestCase, LoaderModuleMockMixin):
    """
    This class contains a set of functions that test salt.modules.zfs module
    """

    def setup_loader_modules(self):
        self.opts = opts = salt.config.DEFAULT_MINION_OPTS.copy()
        self.utils_patch = ZFSMockData().get_patched_utils()
        for key in ("opts", "utils_patch"):
            self.addCleanup(delattr, self, key)

        utils = salt.loader.utils(
            opts, whitelist=["zfs", "args", "systemd", "path", "platform"]
        )
        zfs_obj = {zfs: {"__opts__": opts, "__utils__": utils}}

        return zfs_obj

    def test_exists_success(self):
        """
        Tests successful return of exists function
        """
        ret = {}
        ret["stdout"] = (
            "NAME        USED  AVAIL  REFER  MOUNTPOINT\nmyzpool/mydataset    30K  "
            " 157G    30K  /myzpool/mydataset"
        )
        ret["stderr"] = ""
        ret["retcode"] = 0
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zfs.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zfs.__utils__, self.utils_patch
        ):
            self.assertTrue(zfs.exists("myzpool/mydataset"))

    def test_exists_failure_not_exists(self):
        """
        Tests unsuccessful return of exists function if dataset does not exist
        """
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = "cannot open 'myzpool/mydataset': dataset does not exist"
        ret["retcode"] = 1
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zfs.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zfs.__utils__, self.utils_patch
        ):
            self.assertFalse(zfs.exists("myzpool/mydataset"))

    def test_exists_failure_invalid_name(self):
        """
        Tests unsuccessful return of exists function if dataset name is invalid
        """
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = "cannot open 'myzpool/': invalid dataset name"
        ret["retcode"] = 1
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zfs.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zfs.__utils__, self.utils_patch
        ):
            self.assertFalse(zfs.exists("myzpool/"))

    def test_create_success(self):
        """
        Tests successful return of create function on ZFS file system creation
        """
        res = OrderedDict([("created", True)])
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = ""
        ret["retcode"] = 0
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zfs.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zfs.__utils__, self.utils_patch
        ):
            self.assertEqual(res, zfs.create("myzpool/mydataset"))

    def test_create_success_with_create_parent(self):
        """
        Tests successful return of create function when ``create_parent=True``
        """
        res = OrderedDict([("created", True)])
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = ""
        ret["retcode"] = 0
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zfs.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zfs.__utils__, self.utils_patch
        ):
            self.assertEqual(
                res, zfs.create("myzpool/mydataset/mysubdataset", create_parent=True)
            )

    def test_create_success_with_properties(self):
        """
        Tests successful return of create function on ZFS file system creation (with properties)
        """
        res = OrderedDict([("created", True)])
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = ""
        ret["retcode"] = 0
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zfs.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zfs.__utils__, self.utils_patch
        ):
            self.assertEqual(
                res,
                zfs.create(
                    "myzpool/mydataset",
                    properties={"mountpoint": "/export/zfs", "sharenfs": "on"},
                ),
            )

    def test_create_error_missing_dataset(self):
        """
        Tests unsuccessful return of create function if dataset name is missing
        """
        res = OrderedDict(
            [
                ("created", False),
                ("error", "cannot create 'myzpool': missing dataset name"),
            ]
        )
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = "cannot create 'myzpool': missing dataset name"
        ret["retcode"] = 1
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zfs.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zfs.__utils__, self.utils_patch
        ):
            self.assertEqual(res, zfs.create("myzpool"))

    def test_create_error_trailing_slash(self):
        """
        Tests unsuccessful return of create function if trailing slash in name is present
        """
        res = OrderedDict(
            [
                ("created", False),
                ("error", "cannot create 'myzpool/': trailing slash in name"),
            ]
        )
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = "cannot create 'myzpool/': trailing slash in name"
        ret["retcode"] = 1
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zfs.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zfs.__utils__, self.utils_patch
        ):
            self.assertEqual(res, zfs.create("myzpool/"))

    def test_create_error_no_such_pool(self):
        """
        Tests unsuccessful return of create function if the pool is not present
        """
        res = OrderedDict(
            [
                ("created", False),
                ("error", "cannot create 'myzpool/mydataset': no such pool 'myzpool'"),
            ]
        )
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = "cannot create 'myzpool/mydataset': no such pool 'myzpool'"
        ret["retcode"] = 1
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zfs.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zfs.__utils__, self.utils_patch
        ):
            self.assertEqual(res, zfs.create("myzpool/mydataset"))

    def test_create_error_missing_parent(self):
        """
        Tests unsuccessful return of create function if the parent datasets do not exist
        """
        res = OrderedDict(
            [
                ("created", False),
                (
                    "error",
                    "cannot create 'myzpool/mydataset/mysubdataset': parent does not"
                    " exist",
                ),
            ]
        )
        ret = {}
        ret["stdout"] = ""
        ret[
            "stderr"
        ] = "cannot create 'myzpool/mydataset/mysubdataset': parent does not exist"
        ret["retcode"] = 1
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zfs.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zfs.__utils__, self.utils_patch
        ):
            self.assertEqual(res, zfs.create("myzpool/mydataset/mysubdataset"))

    def test_destroy_success(self):
        """
        Tests successful return of destroy function on ZFS file system destruction
        """
        res = OrderedDict([("destroyed", True)])
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = ""
        ret["retcode"] = 0
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zfs.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zfs.__utils__, self.utils_patch
        ):
            self.assertEqual(res, zfs.destroy("myzpool/mydataset"))

    def test_destroy_error_not_exists(self):
        """
        Tests failure return of destroy function on ZFS file system destruction
        """
        res = OrderedDict(
            [
                ("destroyed", False),
                ("error", "cannot open 'myzpool/mydataset': dataset does not exist"),
            ]
        )
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = "cannot open 'myzpool/mydataset': dataset does not exist"
        ret["retcode"] = 1
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zfs.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zfs.__utils__, self.utils_patch
        ):
            self.assertEqual(res, zfs.destroy("myzpool/mydataset"))

    def test_destroy_error_has_children(self):
        """
        Tests failure return of destroy function on ZFS file system destruction
        """
        res = OrderedDict(
            [
                ("destroyed", False),
                (
                    "error",
                    "\n".join(
                        [
                            "cannot destroy 'myzpool/mydataset': filesystem has"
                            " children",
                            "use 'recursive=True' to destroy the following datasets:",
                            "myzpool/mydataset@snapshot",
                        ]
                    ),
                ),
            ]
        )
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = "\n".join(
            [
                "cannot destroy 'myzpool/mydataset': filesystem has children",
                "use '-r' to destroy the following datasets:",
                "myzpool/mydataset@snapshot",
            ]
        )
        ret["retcode"] = 1
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zfs.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zfs.__utils__, self.utils_patch
        ):
            self.assertEqual(res, zfs.destroy("myzpool/mydataset"))

    @pytest.mark.slow_test
    def test_rename_success(self):
        """
        Tests successful return of rename function
        """
        res = OrderedDict([("renamed", True)])
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = ""
        ret["retcode"] = 0
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zfs.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zfs.__utils__, self.utils_patch
        ):
            self.assertEqual(res, zfs.rename("myzpool/mydataset", "myzpool/newdataset"))

    def test_rename_error_not_exists(self):
        """
        Tests failure return of rename function
        """
        res = OrderedDict(
            [
                ("renamed", False),
                ("error", "cannot open 'myzpool/mydataset': dataset does not exist"),
            ]
        )
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = "cannot open 'myzpool/mydataset': dataset does not exist"
        ret["retcode"] = 1
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zfs.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zfs.__utils__, self.utils_patch
        ):
            self.assertEqual(res, zfs.rename("myzpool/mydataset", "myzpool/newdataset"))

    def test_list_success(self):
        """
        Tests zfs list
        """
        res = OrderedDict(
            [
                (
                    "myzpool",
                    OrderedDict(
                        [
                            ("used", 849329782784),
                            ("avail", 1081258016768),
                            ("refer", 98304),
                            ("mountpoint", "/myzpool"),
                        ]
                    ),
                ),
            ]
        )
        ret = {}
        ret["retcode"] = 0
        ret["stdout"] = "myzpool\t791G\t1007G\t96K\t/myzpool"
        ret["stderr"] = ""
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zfs.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zfs.__utils__, self.utils_patch
        ):
            self.assertEqual(res, zfs.list_("myzpool"))

    @pytest.mark.slow_test
    def test_list_parsable_success(self):
        """
        Tests zfs list with parsable set to False
        """
        res = OrderedDict(
            [
                (
                    "myzpool",
                    OrderedDict(
                        [
                            ("used", "791G"),
                            ("avail", "1007G"),
                            ("refer", "96K"),
                            ("mountpoint", "/myzpool"),
                        ]
                    ),
                ),
            ]
        )
        ret = {}
        ret["retcode"] = 0
        ret["stdout"] = "myzpool\t791G\t1007G\t96K\t/myzpool"
        ret["stderr"] = ""
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zfs.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zfs.__utils__, self.utils_patch
        ):
            self.assertEqual(res, zfs.list_("myzpool", parsable=False))

    def test_list_custom_success(self):
        """
        Tests zfs list
        """
        res = OrderedDict(
            [
                (
                    "myzpool",
                    OrderedDict(
                        [
                            ("canmount", True),
                            ("used", 849329782784),
                            ("avail", 1081258016768),
                            ("compression", False),
                        ]
                    ),
                ),
            ]
        )
        ret = {}
        ret["retcode"] = 0
        ret["stdout"] = "myzpool\ton\t791G\t1007G\toff"
        ret["stderr"] = ""
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zfs.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zfs.__utils__, self.utils_patch
        ):
            self.assertEqual(
                res, zfs.list_("myzpool", properties="canmount,used,avail,compression")
            )

    def test_list_custom_parsable_success(self):
        """
        Tests zfs list
        """
        res = OrderedDict(
            [
                (
                    "myzpool",
                    OrderedDict(
                        [
                            ("canmount", "on"),
                            ("used", "791G"),
                            ("avail", "1007G"),
                            ("compression", "off"),
                        ]
                    ),
                ),
            ]
        )
        ret = {}
        ret["retcode"] = 0
        ret["stdout"] = "myzpool\ton\t791G\t1007G\toff"
        ret["stderr"] = ""
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zfs.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zfs.__utils__, self.utils_patch
        ):
            self.assertEqual(
                res,
                zfs.list_(
                    "myzpool",
                    properties="canmount,used,avail,compression",
                    parsable=False,
                ),
            )

    def test_list_error_no_dataset(self):
        """
        Tests zfs list
        """
        res = OrderedDict()
        ret = {}
        ret["retcode"] = 1
        ret["stdout"] = "cannot open 'myzpool': dataset does not exist"
        ret["stderr"] = ""
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zfs.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zfs.__utils__, self.utils_patch
        ):
            self.assertEqual(res, zfs.list_("myzpool"))

    @pytest.mark.slow_test
    def test_list_mount_success(self):
        """
        Tests zfs list_mount
        """
        res = OrderedDict(
            [("myzpool/data", "/data"), ("myzpool/data/ares", "/data/ares")]
        )
        ret = {}
        ret["retcode"] = 0
        ret["stdout"] = "\n".join(
            ["myzpool/data\t\t\t\t/data", "myzpool/data/ares\t\t\t/data/ares"]
        )
        ret["stderr"] = ""
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zfs.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zfs.__utils__, self.utils_patch
        ):
            self.assertEqual(res, zfs.list_mount())

    def test_mount_success(self):
        """
        Tests zfs mount of filesystem
        """
        res = OrderedDict([("mounted", True)])
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = ""
        ret["retcode"] = 0
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zfs.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zfs.__utils__, self.utils_patch
        ):
            self.assertEqual(res, zfs.mount("myzpool/mydataset"))

    def test_mount_failure(self):
        """
        Tests zfs mount of already mounted filesystem
        """
        res = OrderedDict(
            [
                ("mounted", False),
                (
                    "error",
                    "cannot mount 'myzpool/mydataset': filesystem already mounted",
                ),
            ]
        )
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = "cannot mount 'myzpool/mydataset': filesystem already mounted"
        ret["retcode"] = 1
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zfs.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zfs.__utils__, self.utils_patch
        ):
            self.assertEqual(res, zfs.mount("myzpool/mydataset"))

    def test_unmount_success(self):
        """
        Tests zfs unmount of filesystem
        """
        res = OrderedDict([("unmounted", True)])
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = ""
        ret["retcode"] = 0
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zfs.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zfs.__utils__, self.utils_patch
        ):
            self.assertEqual(res, zfs.unmount("myzpool/mydataset"))

    def test_unmount_failure(self):
        """
        Tests zfs unmount of already mounted filesystem
        """
        res = OrderedDict(
            [
                ("unmounted", False),
                ("error", "cannot mount 'myzpool/mydataset': not currently mounted"),
            ]
        )
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = "cannot mount 'myzpool/mydataset': not currently mounted"
        ret["retcode"] = 1
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zfs.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zfs.__utils__, self.utils_patch
        ):
            self.assertEqual(res, zfs.unmount("myzpool/mydataset"))

    def test_inherit_success(self):
        """
        Tests zfs inherit of compression property
        """
        res = OrderedDict([("inherited", True)])
        ret = {"pid": 45193, "retcode": 0, "stderr": "", "stdout": ""}
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zfs.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zfs.__utils__, self.utils_patch
        ):
            self.assertEqual(res, zfs.inherit("compression", "myzpool/mydataset"))

    def test_inherit_failure(self):
        """
        Tests zfs inherit of canmount
        """
        res = OrderedDict(
            [
                ("inherited", False),
                ("error", "'canmount' property cannot be inherited"),
            ]
        )
        ret = {
            "pid": 43898,
            "retcode": 1,
            "stderr": "'canmount' property cannot be inherited",
            "stdout": "",
        }
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zfs.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zfs.__utils__, self.utils_patch
        ):
            self.assertEqual(res, zfs.inherit("canmount", "myzpool/mydataset"))

    @pytest.mark.slow_test
    def test_diff(self):
        """
        Tests zfs diff
        """
        res = [
            "1517063879.144517494\tM\t\t/data/test/",
            "1517063875.296592355\t+\t\t/data/test/world",
            "1517063879.274438467\t+\t\t/data/test/hello",
        ]
        ret = {}
        ret["retcode"] = 0
        ret["stdout"] = "\n".join(
            [
                "1517063879.144517494\tM\t\t/data/test/",
                "1517063875.296592355\t+\t\t/data/test/world",
                "1517063879.274438467\t+\t\t/data/test/hello",
            ]
        )
        ret["stderr"] = ""
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zfs.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zfs.__utils__, self.utils_patch
        ):
            self.assertEqual(
                res, zfs.diff("myzpool/mydataset@yesterday", "myzpool/mydataset")
            )

    def test_diff_parsed_time(self):
        """
        Tests zfs diff
        """
        ## NOTE: do not hardcode parsed timestamps, timezone play a role here.
        ##       zfs diff output seems to be timezone aware
        res = OrderedDict(
            [
                (
                    strftime(1517063879.144517494, "%Y-%m-%d.%H:%M:%S.%f"),
                    "M\t\t/data/test/",
                ),
                (
                    strftime(1517063875.296592355, "%Y-%m-%d.%H:%M:%S.%f"),
                    "+\t\t/data/test/world",
                ),
                (
                    strftime(1517063879.274438467, "%Y-%m-%d.%H:%M:%S.%f"),
                    "+\t\t/data/test/hello",
                ),
            ]
        )
        ret = {}
        ret["retcode"] = 0
        ret["stdout"] = "\n".join(
            [
                "1517063879.144517494\tM\t\t/data/test/",
                "1517063875.296592355\t+\t\t/data/test/world",
                "1517063879.274438467\t+\t\t/data/test/hello",
            ]
        )
        ret["stderr"] = ""
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zfs.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zfs.__utils__, self.utils_patch
        ):
            self.assertEqual(
                res, zfs.diff("myzpool/data@yesterday", "myzpool/data", parsable=False)
            )

    @pytest.mark.slow_test
    def test_rollback_success(self):
        """
        Tests zfs rollback success
        """
        res = OrderedDict([("rolledback", True)])
        ret = {"pid": 56502, "retcode": 0, "stderr": "", "stdout": ""}
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zfs.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zfs.__utils__, self.utils_patch
        ):
            self.assertEqual(res, zfs.rollback("myzpool/mydataset@yesterday"))

    @pytest.mark.slow_test
    def test_rollback_failure(self):
        """
        Tests zfs rollback failure
        """
        res = OrderedDict(
            [
                ("rolledback", False),
                (
                    "error",
                    "\n".join(
                        [
                            "cannot rollback to 'myzpool/mydataset@yesterday': more"
                            " recent snapshots or bookmarks exist",
                            "use 'recursive=True' to force deletion of the following"
                            " snapshots and bookmarks:",
                            "myzpool/mydataset@today",
                        ]
                    ),
                ),
            ]
        )
        ret = {
            "pid": 57471,
            "retcode": 1,
            "stderr": (
                "cannot rollback to 'myzpool/mydataset@yesterday': more recent"
                " snapshots or bookmarks exist\nuse '-r' to force deletion of the"
                " following snapshots and bookmarks:\nmyzpool/mydataset@today"
            ),
            "stdout": "",
        }
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zfs.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zfs.__utils__, self.utils_patch
        ):
            self.assertEqual(res, zfs.rollback("myzpool/mydataset@yesterday"))

    def test_clone_success(self):
        """
        Tests zfs clone success
        """
        res = OrderedDict([("cloned", True)])
        ret = {"pid": 64532, "retcode": 0, "stderr": "", "stdout": ""}
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zfs.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zfs.__utils__, self.utils_patch
        ):
            self.assertEqual(
                res, zfs.clone("myzpool/mydataset@yesterday", "myzpool/yesterday")
            )

    def test_clone_failure(self):
        """
        Tests zfs clone failure
        """
        res = OrderedDict(
            [
                ("cloned", False),
                (
                    "error",
                    "cannot create 'myzpool/archive/yesterday': parent does not exist",
                ),
            ]
        )
        ret = {
            "pid": 64864,
            "retcode": 1,
            "stderr": (
                "cannot create 'myzpool/archive/yesterday': parent does not exist"
            ),
            "stdout": "",
        }
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zfs.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zfs.__utils__, self.utils_patch
        ):
            self.assertEqual(
                res,
                zfs.clone("myzpool/mydataset@yesterday", "myzpool/archive/yesterday"),
            )

    @pytest.mark.slow_test
    def test_promote_success(self):
        """
        Tests zfs promote success
        """
        res = OrderedDict([("promoted", True)])
        ret = {"pid": 69075, "retcode": 0, "stderr": "", "stdout": ""}
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zfs.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zfs.__utils__, self.utils_patch
        ):
            self.assertEqual(res, zfs.promote("myzpool/yesterday"))

    def test_promote_failure(self):
        """
        Tests zfs promote failure
        """
        res = OrderedDict(
            [
                ("promoted", False),
                (
                    "error",
                    "cannot promote 'myzpool/yesterday': not a cloned filesystem",
                ),
            ]
        )
        ret = {
            "pid": 69209,
            "retcode": 1,
            "stderr": "cannot promote 'myzpool/yesterday': not a cloned filesystem",
            "stdout": "",
        }
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zfs.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zfs.__utils__, self.utils_patch
        ):
            self.assertEqual(res, zfs.promote("myzpool/yesterday"))

    @pytest.mark.slow_test
    def test_bookmark_success(self):
        """
        Tests zfs bookmark success
        """
        with patch("salt.utils.path.which", MagicMock(return_value="/usr/bin/man")):
            res = OrderedDict([("bookmarked", True)])
            ret = {"pid": 20990, "retcode": 0, "stderr": "", "stdout": ""}
            mock_cmd = MagicMock(return_value=ret)
            with patch.dict(zfs.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
                zfs.__utils__, self.utils_patch
            ):
                self.assertEqual(
                    res,
                    zfs.bookmark(
                        "myzpool/mydataset@yesterday", "myzpool/mydataset#important"
                    ),
                )

    @pytest.mark.slow_test
    def test_holds_success(self):
        """
        Tests zfs holds success
        """
        res = OrderedDict(
            [
                ("important", "Wed Dec 23 21:06 2015"),
                ("release-1.0", "Wed Dec 23 21:08 2015"),
            ]
        )
        ret = {
            "pid": 40216,
            "retcode": 0,
            "stderr": "",
            "stdout": (
                "myzpool/mydataset@baseline\timportant  \tWed Dec 23 21:06"
                " 2015\nmyzpool/mydataset@baseline\trelease-1.0\tWed Dec 23 21:08 2015"
            ),
        }
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zfs.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zfs.__utils__, self.utils_patch
        ):
            self.assertEqual(res, zfs.holds("myzpool/mydataset@baseline"))

    def test_holds_failure(self):
        """
        Tests zfs holds failure
        """
        res = OrderedDict(
            [
                (
                    "error",
                    "cannot open 'myzpool/mydataset@baseline': dataset does not exist",
                ),
            ]
        )
        ret = {
            "pid": 40993,
            "retcode": 1,
            "stderr": (
                "cannot open 'myzpool/mydataset@baseline': dataset does not exist"
            ),
            "stdout": "no datasets available",
        }
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zfs.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zfs.__utils__, self.utils_patch
        ):
            self.assertEqual(res, zfs.holds("myzpool/mydataset@baseline"))

    def test_hold_success(self):
        """
        Tests zfs hold success
        """
        res = OrderedDict([("held", True)])
        ret = {"pid": 50876, "retcode": 0, "stderr": "", "stdout": ""}
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zfs.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zfs.__utils__, self.utils_patch
        ):
            self.assertEqual(
                res,
                zfs.hold(
                    "important",
                    "myzpool/mydataset@baseline",
                    "myzpool/mydataset@release-1.0",
                ),
            )

    def test_hold_failure(self):
        """
        Tests zfs hold failure
        """
        res = OrderedDict(
            [
                ("held", False),
                (
                    "error",
                    "cannot hold snapshot 'myzpool/mydataset@baseline': tag already"
                    " exists on this dataset",
                ),
            ]
        )
        ret = {
            "pid": 51006,
            "retcode": 1,
            "stderr": (
                "cannot hold snapshot 'myzpool/mydataset@baseline': tag already exists"
                " on this dataset"
            ),
            "stdout": "",
        }
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zfs.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zfs.__utils__, self.utils_patch
        ):
            self.assertEqual(res, zfs.hold("important", "myzpool/mydataset@baseline"))

    def test_release_success(self):
        """
        Tests zfs release success
        """
        res = OrderedDict([("released", True)])
        ret = {"pid": 50876, "retcode": 0, "stderr": "", "stdout": ""}
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zfs.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zfs.__utils__, self.utils_patch
        ):
            self.assertEqual(
                res,
                zfs.release(
                    "important",
                    "myzpool/mydataset@baseline",
                    "myzpool/mydataset@release-1.0",
                ),
            )

    @pytest.mark.slow_test
    def test_release_failure(self):
        """
        Tests zfs release failure
        """
        res = OrderedDict(
            [
                ("released", False),
                (
                    "error",
                    "cannot release hold from snapshot 'myzpool/mydataset@baseline': no"
                    " such tag on this dataset",
                ),
            ]
        )
        ret = {
            "pid": 51006,
            "retcode": 1,
            "stderr": (
                "cannot release hold from snapshot 'myzpool/mydataset@baseline': no"
                " such tag on this dataset"
            ),
            "stdout": "",
        }
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zfs.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zfs.__utils__, self.utils_patch
        ):
            self.assertEqual(
                res, zfs.release("important", "myzpool/mydataset@baseline")
            )

    def test_snapshot_success(self):
        """
        Tests zfs snapshot success
        """
        res = OrderedDict([("snapshotted", True)])
        ret = {"pid": 69125, "retcode": 0, "stderr": "", "stdout": ""}
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zfs.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zfs.__utils__, self.utils_patch
        ):
            self.assertEqual(res, zfs.snapshot("myzpool/mydataset@baseline"))

    def test_snapshot_failure(self):
        """
        Tests zfs snapshot failure
        """
        res = OrderedDict(
            [
                ("snapshotted", False),
                (
                    "error",
                    "cannot create snapshot 'myzpool/mydataset@baseline': dataset"
                    " already exists",
                ),
            ]
        )
        ret = {
            "pid": 68526,
            "retcode": 1,
            "stderr": (
                "cannot create snapshot 'myzpool/mydataset@baseline': dataset already"
                " exists"
            ),
            "stdout": "",
        }
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zfs.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zfs.__utils__, self.utils_patch
        ):
            self.assertEqual(res, zfs.snapshot("myzpool/mydataset@baseline"))

    def test_snapshot_failure2(self):
        """
        Tests zfs snapshot failure
        """
        res = OrderedDict(
            [
                ("snapshotted", False),
                ("error", "cannot open 'myzpool/mydataset': dataset does not exist"),
            ]
        )
        ret = {
            "pid": 69256,
            "retcode": 2,
            "stderr": (
                "cannot open 'myzpool/mydataset': dataset does not"
                " exist\nusage:\n\tsnapshot [-r] [-o property=value] ..."
                " <filesystem|volume>@<snap> ...\n\nFor the property list, run: zfs"
                " set|get\n\nFor the delegated permission list, run: zfs allow|unallow"
            ),
            "stdout": "",
        }
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zfs.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zfs.__utils__, self.utils_patch
        ):
            self.assertEqual(res, zfs.snapshot("myzpool/mydataset@baseline"))

    def test_set_success(self):
        """
        Tests zfs set success
        """
        res = OrderedDict([("set", True)])
        ret = {"pid": 79736, "retcode": 0, "stderr": "", "stdout": ""}
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zfs.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zfs.__utils__, self.utils_patch
        ):
            self.assertEqual(res, zfs.set("myzpool/mydataset", compression="lz4"))

    @pytest.mark.slow_test
    def test_set_failure(self):
        """
        Tests zfs set failure
        """
        res = OrderedDict(
            [
                ("set", False),
                (
                    "error",
                    "cannot set property for 'myzpool/mydataset': 'canmount' must be"
                    " one of 'on | off | noauto'",
                ),
            ]
        )
        ret = {
            "pid": 79887,
            "retcode": 1,
            "stderr": (
                "cannot set property for 'myzpool/mydataset': 'canmount' must be one of"
                " 'on | off | noauto'"
            ),
            "stdout": "",
        }
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zfs.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zfs.__utils__, self.utils_patch
        ):
            self.assertEqual(res, zfs.set("myzpool/mydataset", canmount="lz4"))

    def test_get_success(self):
        """
        Tests zfs get success
        """
        res = OrderedDict(
            [
                (
                    "myzpool",
                    OrderedDict([("used", OrderedDict([("value", 906238099456)]))]),
                ),
            ]
        )
        ret = {
            "pid": 562,
            "retcode": 0,
            "stderr": "",
            "stdout": "myzpool\tused\t906238099456",
        }
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zfs.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zfs.__utils__, self.utils_patch
        ):
            self.assertEqual(res, zfs.get("myzpool", properties="used", fields="value"))

    def test_get_parsable_success(self):
        """
        Tests zfs get with parsable output
        """
        res = OrderedDict(
            [("myzpool", OrderedDict([("used", OrderedDict([("value", "844G")]))]))]
        )
        ret = {
            "pid": 562,
            "retcode": 0,
            "stderr": "",
            "stdout": "myzpool\tused\t906238099456",
        }
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zfs.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zfs.__utils__, self.utils_patch
        ):
            self.assertEqual(
                res,
                zfs.get("myzpool", properties="used", fields="value", parsable=False),
            )



import salt.modules.helm as helm

# Import Exception
from salt.exceptions import CommandExecutionError
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, call, patch
from tests.support.unit import TestCase


class HelmTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.modules.helm
    """

    def setup_loader_modules(self):
        return {helm: {}}

    def test__prepare_cmd(self):
        self.assertEqual(helm._prepare_cmd(), ("helm",))

    def test__prepare_cmd_binary(self):
        self.assertEqual(helm._prepare_cmd(binary="binary"), ("binary",))

    def test__prepare_cmd_commands(self):
        self.assertEqual(
            helm._prepare_cmd(commands=["com1", "com2"]),
            (
                "helm",
                "com1",
                "com2",
            ),
        )

    def test__prepare_cmd_flags(self):
        self.assertEqual(
            helm._prepare_cmd(flags=["flag1", "--flag2"]),
            (
                "helm",
                "--flag1",
                "--flag2",
            ),
        )

    def test__prepare_cmd_kvflags(self):
        result_tuple = helm._prepare_cmd(
            kvflags={"kflag1": "vflag1", "--kflag2": "vflag2"}
        )
        tuple_valide_1 = (
            "helm",
            "--kflag1",
            "vflag1",
            "--kflag2",
            "vflag2",
        )
        tuple_valide_2 = (
            "helm",
            "--kflag2",
            "vflag2",
            "--kflag1",
            "vflag1",
        )

        #
        self.assertEqual(
            True, result_tuple == tuple_valide_1 or result_tuple == tuple_valide_2
        )

    def test__exec_cmd(self):
        cmd_prepare = helm._prepare_cmd()
        cmd_prepare_str = " ".join(cmd_prepare)
        cmd_return = {
            "stdout": "succes",
            "stderr": "",
            "retcode": 0,
        }
        result = cmd_return
        result.update({"cmd": cmd_prepare_str})
        with patch.dict(
            helm.__salt__,
            {  # pylint: disable=no-member
                "cmd.run_all": MagicMock(return_value=cmd_return)
            },
        ):
            self.assertEqual(helm._exec_cmd(), result)

    def test__exec_true_return_valid(self):
        _exec_cmd_return = {"retcode": 0}
        with patch(
            "salt.modules.helm._exec_cmd", MagicMock(return_value=_exec_cmd_return)
        ):
            self.assertEqual(True, helm._exec_true_return())

    def test__exec_true_return_not_valid(self):
        _exec_cmd_return = {"retcode": -1, "stderr": "test"}
        with patch(
            "salt.modules.helm._exec_cmd", MagicMock(return_value=_exec_cmd_return)
        ):
            self.assertEqual("test", helm._exec_true_return())

    def test__exec_string_return_valid(self):
        _exec_cmd_return = {"retcode": 0, "stdout": "test"}
        with patch(
            "salt.modules.helm._exec_cmd", MagicMock(return_value=_exec_cmd_return)
        ):
            self.assertEqual("test", helm._exec_string_return())

    def test__exec_string_return_not_valid(self):
        _exec_cmd_return = {"retcode": -1, "stderr": "test"}
        with patch(
            "salt.modules.helm._exec_cmd", MagicMock(return_value=_exec_cmd_return)
        ):
            self.assertEqual("test", helm._exec_string_return())

    def test__exec_dict_return_valide(self):
        _exec_cmd_return = {"retcode": 0, "stdout": '{"test": true}'}
        with patch(
            "salt.modules.helm._exec_cmd", MagicMock(return_value=_exec_cmd_return)
        ):
            self.assertEqual({"test": True}, helm._exec_dict_return())

    def test__exec_dict_return_valide_no_json(self):
        _exec_cmd_return = {"retcode": 0, "stdout": '{"test": true}'}
        with patch(
            "salt.modules.helm._exec_cmd", MagicMock(return_value=_exec_cmd_return)
        ):
            self.assertEqual(
                '{"test": true}', helm._exec_dict_return(kvflags={"output": "table"})
            )

    def test__exec_dict_return_not_valid(self):
        _exec_cmd_return = {"retcode": -1, "stderr": "test"}
        with patch(
            "salt.modules.helm._exec_cmd", MagicMock(return_value=_exec_cmd_return)
        ):
            self.assertEqual("test", helm._exec_dict_return())

    def test_completion(self):
        magic_mock = MagicMock(return_value="the_return")
        with patch("salt.modules.helm._exec_string_return", magic_mock):
            self.assertEqual("the_return", helm.completion("bash"))
            self.assertEqual(
                [call(commands=["completion", "bash"], flags=None, kvflags=None)],
                magic_mock.mock_calls,
            )

    def test_create(self):
        magic_mock = MagicMock(return_value=True)
        with patch("salt.modules.helm._exec_true_return", magic_mock):
            self.assertEqual(True, helm.create("name"))
            self.assertEqual(
                [call(commands=["create", "name"], flags=None, kvflags=None)],
                magic_mock.mock_calls,
            )

    def test_dependency_build(self):
        magic_mock = MagicMock(return_value=True)
        with patch("salt.modules.helm._exec_true_return", magic_mock):
            self.assertEqual(True, helm.dependency_build("chart"))
            self.assertEqual(
                [
                    call(
                        commands=["dependency", "build", "chart"],
                        flags=None,
                        kvflags=None,
                    )
                ],
                magic_mock.mock_calls,
            )

    def test_dependency_list(self):
        magic_mock = MagicMock(return_value="the_return")
        with patch("salt.modules.helm._exec_string_return", magic_mock):
            self.assertEqual("the_return", helm.dependency_list("chart"))
            self.assertEqual(
                [
                    call(
                        commands=["dependency", "list", "chart"],
                        flags=None,
                        kvflags=None,
                    )
                ],
                magic_mock.mock_calls,
            )

    def test_dependency_update(self):
        magic_mock = MagicMock(return_value=True)
        with patch("salt.modules.helm._exec_true_return", magic_mock):
            self.assertEqual(True, helm.dependency_update("chart"))
            self.assertEqual(
                [
                    call(
                        commands=["dependency", "update", "chart"],
                        flags=None,
                        kvflags=None,
                    )
                ],
                magic_mock.mock_calls,
            )

    def test_env(self):
        magic_mock = MagicMock(return_value="the_return")
        with patch("salt.modules.helm._exec_string_return", magic_mock):
            self.assertEqual("the_return", helm.env())
            self.assertEqual(
                [call(commands=["env"], flags=None, kvflags=None)],
                magic_mock.mock_calls,
            )

    def test_get_all(self):
        magic_mock = MagicMock(return_value="the_return")
        with patch("salt.modules.helm._exec_string_return", magic_mock):
            self.assertEqual("the_return", helm.get_all("release"))
            self.assertEqual(
                [call(commands=["get", "all", "release"], flags=None, kvflags=None)],
                magic_mock.mock_calls,
            )

    def test_get_hooks(self):
        magic_mock = MagicMock(return_value="the_return")
        with patch("salt.modules.helm._exec_string_return", magic_mock):
            self.assertEqual("the_return", helm.get_hooks("release"))
            self.assertEqual(
                [call(commands=["get", "hooks", "release"], flags=None, kvflags=None)],
                magic_mock.mock_calls,
            )

    def test_get_manifest(self):
        magic_mock = MagicMock(return_value="the_return")
        with patch("salt.modules.helm._exec_string_return", magic_mock):
            self.assertEqual("the_return", helm.get_manifest("release"))
            self.assertEqual(
                [
                    call(
                        commands=["get", "manifest", "release"],
                        flags=None,
                        kvflags=None,
                    )
                ],
                magic_mock.mock_calls,
            )

    def test_get_notes(self):
        magic_mock = MagicMock(return_value="the_return")
        with patch("salt.modules.helm._exec_string_return", magic_mock):
            self.assertEqual("the_return", helm.get_notes("release"))
            self.assertEqual(
                [call(commands=["get", "notes", "release"], flags=None, kvflags=None)],
                magic_mock.mock_calls,
            )

    def test_get_values(self):
        magic_mock = MagicMock(return_value={"test": True})
        with patch("salt.modules.helm._exec_dict_return", magic_mock):
            self.assertEqual({"test": True}, helm.get_values("release"))
            self.assertEqual(
                [call(commands=["get", "values", "release"], flags=None, kvflags=None)],
                magic_mock.mock_calls,
            )

    def test_help_(self):
        magic_mock = MagicMock(return_value="the_return")
        with patch("salt.modules.helm._exec_string_return", magic_mock):
            self.assertEqual("the_return", helm.help_("command"))
            self.assertEqual(
                [call(commands=["help", "command"], flags=None, kvflags=None)],
                magic_mock.mock_calls,
            )

    def test_history(self):
        magic_mock = MagicMock(return_value={"test": True})
        with patch("salt.modules.helm._exec_dict_return", magic_mock):
            self.assertEqual({"test": True}, helm.history("release"))
            self.assertEqual(
                [call(commands=["history", "release"], flags=None, kvflags=None)],
                magic_mock.mock_calls,
            )

    def test_install(self):
        magic_mock = MagicMock(return_value=True)
        with patch("salt.modules.helm._exec_true_return", magic_mock):
            self.assertEqual(True, helm.install("release", "chart"))
            self.assertEqual(
                [
                    call(
                        commands=["install", "release", "chart"],
                        flags=None,
                        kvflags=None,
                    )
                ],
                magic_mock.mock_calls,
            )

    def test_lint(self):
        magic_mock = MagicMock(return_value=True)
        with patch("salt.modules.helm._exec_true_return", magic_mock):
            self.assertEqual(True, helm.lint("path"))
            self.assertEqual(
                [call(commands=["lint", "path"], flags=None, kvflags=None)],
                magic_mock.mock_calls,
            )

    def test_list_(self):
        magic_mock = MagicMock(return_value={"test": True})
        with patch("salt.modules.helm._exec_dict_return", magic_mock):
            self.assertEqual({"test": True}, helm.list_())
            self.assertEqual(
                [call(commands=["list"], flags=None, kvflags=None)],
                magic_mock.mock_calls,
            )

    def test_package(self):
        magic_mock = MagicMock(return_value=True)
        with patch("salt.modules.helm._exec_true_return", magic_mock):
            self.assertEqual(True, helm.package("chart"))
            self.assertEqual(
                [call(commands=["package", "chart"], flags=None, kvflags=None)],
                magic_mock.mock_calls,
            )

    def test_plugin_install(self):
        magic_mock = MagicMock(return_value=True)
        with patch("salt.modules.helm._exec_true_return", magic_mock):
            self.assertEqual(True, helm.plugin_install("path"))
            self.assertEqual(
                [
                    call(
                        commands=["plugin", "install", "path"], flags=None, kvflags=None
                    )
                ],
                magic_mock.mock_calls,
            )

    def test_plugin_list(self):
        magic_mock = MagicMock(return_value="the_return")
        with patch("salt.modules.helm._exec_string_return", magic_mock):
            self.assertEqual("the_return", helm.plugin_list())
            self.assertEqual(
                [call(commands=["plugin", "list"], flags=None, kvflags=None)],
                magic_mock.mock_calls,
            )

    def test_plugin_uninstall(self):
        magic_mock = MagicMock(return_value=True)
        with patch("salt.modules.helm._exec_true_return", magic_mock):
            self.assertEqual(True, helm.plugin_uninstall("plugin"))
            self.assertEqual(
                [
                    call(
                        commands=["plugin", "uninstall", "plugin"],
                        flags=None,
                        kvflags=None,
                    )
                ],
                magic_mock.mock_calls,
            )

    def test_plugin_update(self):
        magic_mock = MagicMock(return_value=True)
        with patch("salt.modules.helm._exec_true_return", magic_mock):
            self.assertEqual(True, helm.plugin_update("plugin"))
            self.assertEqual(
                [
                    call(
                        commands=["plugin", "update", "plugin"],
                        flags=None,
                        kvflags=None,
                    )
                ],
                magic_mock.mock_calls,
            )

    def test_pull(self):
        magic_mock = MagicMock(return_value=True)
        with patch("salt.modules.helm._exec_true_return", magic_mock):
            self.assertEqual(True, helm.pull("pkg"))
            self.assertEqual(
                [call(commands=["pull", "pkg"], flags=None, kvflags=None)],
                magic_mock.mock_calls,
            )

    def test_repo_add(self):
        magic_mock = MagicMock(return_value=True)
        with patch("salt.modules.helm._exec_true_return", magic_mock):
            self.assertEqual(True, helm.repo_add("name", "url"))
            self.assertEqual(
                [
                    call(
                        commands=["repo", "add", "name", "url"],
                        flags=None,
                        kvflags=None,
                    )
                ],
                magic_mock.mock_calls,
            )

    def test_repo_index(self):
        magic_mock = MagicMock(return_value=True)
        with patch("salt.modules.helm._exec_true_return", magic_mock):
            self.assertEqual(True, helm.repo_index("directory"))
            self.assertEqual(
                [
                    call(
                        commands=["repo", "index", "directory"],
                        flags=None,
                        kvflags=None,
                    )
                ],
                magic_mock.mock_calls,
            )

    def test_repo_list(self):
        magic_mock = MagicMock(return_value={"test": True})
        with patch("salt.modules.helm._exec_dict_return", magic_mock):
            self.assertEqual({"test": True}, helm.repo_list())
            self.assertEqual(
                [call(commands=["repo", "list"], flags=None, kvflags=None)],
                magic_mock.mock_calls,
            )

    def test_repo_remove(self):
        magic_mock = MagicMock(return_value=True)
        with patch("salt.modules.helm._exec_true_return", magic_mock):
            self.assertEqual(True, helm.repo_remove("name"))
            self.assertEqual(
                [call(commands=["repo", "remove", "name"], flags=None, kvflags=None)],
                magic_mock.mock_calls,
            )

    def test_repo_update(self):
        magic_mock = MagicMock(return_value=True)
        with patch("salt.modules.helm._exec_true_return", magic_mock):
            self.assertEqual(True, helm.repo_update())
            self.assertEqual(
                [call(commands=["repo", "update"], flags=None, kvflags=None)],
                magic_mock.mock_calls,
            )

    def test_repo_manage_present_bad_list(self):
        with patch("salt.modules.helm.repo_list", MagicMock(return_value=None)):
            with self.assertRaises(CommandExecutionError):
                helm.repo_manage(present=["test"])

    def test_repo_manage_present_bad_format(self):
        with patch("salt.modules.helm.repo_list", MagicMock(return_value=None)):
            with self.assertRaises(CommandExecutionError):
                helm.repo_manage(present=[{"test": True}])

    def test_repo_manage_present_failed(self):
        result_wanted = {
            "present": [],
            "added": [],
            "absent": [],
            "removed": [],
            "failed": [{"name": "myname", "url": "myurl"}],
        }
        with patch("salt.modules.helm.repo_list", MagicMock(return_value=None)):
            with patch("salt.modules.helm.repo_add", MagicMock(return_value="failed")):
                self.assertEqual(
                    helm.repo_manage(present=[{"name": "myname", "url": "myurl"}]),
                    result_wanted,
                )

    def test_repo_manage_present_succeed(self):
        result_wanted = {
            "present": [],
            "added": [{"name": "myname", "url": "myurl"}],
            "absent": [],
            "removed": [],
            "failed": [],
        }
        with patch("salt.modules.helm.repo_list", MagicMock(return_value=None)):
            with patch("salt.modules.helm.repo_add", MagicMock(return_value=True)):
                self.assertEqual(
                    helm.repo_manage(present=[{"name": "myname", "url": "myurl"}]),
                    result_wanted,
                )

    def test_repo_manage_present_already_present(self):
        result_wanted = {
            "present": [{"name": "myname", "url": "myurl"}],
            "added": [],
            "absent": [],
            "removed": [],
            "failed": [],
        }
        with patch(
            "salt.modules.helm.repo_list",
            MagicMock(return_value=[{"name": "myname", "url": "myurl"}]),
        ):
            self.assertEqual(
                helm.repo_manage(present=[{"name": "myname", "url": "myurl"}]),
                result_wanted,
            )

    def test_repo_manage_prune(self):
        result_wanted = {
            "present": [],
            "added": [],
            "absent": [],
            "removed": ["myname"],
            "failed": [],
        }
        with patch(
            "salt.modules.helm.repo_list",
            MagicMock(return_value=[{"name": "myname", "url": "myurl"}]),
        ):
            with patch("salt.modules.helm.repo_remove", MagicMock(return_value=True)):
                self.assertEqual(helm.repo_manage(prune=True), result_wanted)

    def test_repo_manage_absent(self):
        result_wanted = {
            "present": [],
            "added": [],
            "absent": ["myname"],
            "removed": [],
            "failed": [],
        }
        with patch("salt.modules.helm.repo_list", MagicMock(return_value=None)):
            with patch("salt.modules.helm.repo_remove", MagicMock(return_value=False)):
                self.assertEqual(helm.repo_manage(absent=["myname"]), result_wanted)

    def test_repo_manage_removed(self):
        result_wanted = {
            "present": [],
            "added": [],
            "absent": [],
            "removed": ["myname"],
            "failed": [],
        }
        with patch("salt.modules.helm.repo_list", MagicMock(return_value=None)):
            with patch("salt.modules.helm.repo_remove", MagicMock(return_value=True)):
                self.assertEqual(helm.repo_manage(absent=["myname"]), result_wanted)

    def test_rollback(self):
        magic_mock = MagicMock(return_value=True)
        with patch("salt.modules.helm._exec_true_return", magic_mock):
            self.assertEqual(True, helm.rollback("release", "revision"))
            self.assertEqual(
                [
                    call(
                        commands=["rollback", "release", "revision"],
                        flags=None,
                        kvflags=None,
                    )
                ],
                magic_mock.mock_calls,
            )

    def test_search_hub(self):
        magic_mock = MagicMock(return_value={"test": True})
        with patch("salt.modules.helm._exec_dict_return", magic_mock):
            self.assertEqual({"test": True}, helm.search_hub("keyword"))
            self.assertEqual(
                [call(commands=["search", "hub", "keyword"], flags=None, kvflags=None)],
                magic_mock.mock_calls,
            )

    def test_search_repo(self):
        magic_mock = MagicMock(return_value={"test": True})
        with patch("salt.modules.helm._exec_dict_return", magic_mock):
            self.assertEqual({"test": True}, helm.search_repo("keyword"))
            self.assertEqual(
                [
                    call(
                        commands=["search", "repo", "keyword"], flags=None, kvflags=None
                    )
                ],
                magic_mock.mock_calls,
            )

    def test_show_all(self):
        magic_mock = MagicMock(return_value="the_return")
        with patch("salt.modules.helm._exec_string_return", magic_mock):
            self.assertEqual("the_return", helm.show_all("chart"))
            self.assertEqual(
                [call(commands=["show", "all", "chart"], flags=None, kvflags=None)],
                magic_mock.mock_calls,
            )

    def test_show_chart(self):
        magic_mock = MagicMock(return_value="the_return")
        with patch("salt.modules.helm._exec_string_return", magic_mock):
            self.assertEqual("the_return", helm.show_chart("chart"))
            self.assertEqual(
                [call(commands=["show", "chart", "chart"], flags=None, kvflags=None)],
                magic_mock.mock_calls,
            )

    def test_show_readme(self):
        magic_mock = MagicMock(return_value="the_return")
        with patch("salt.modules.helm._exec_string_return", magic_mock):
            self.assertEqual("the_return", helm.show_readme("chart"))
            self.assertEqual(
                [call(commands=["show", "readme", "chart"], flags=None, kvflags=None)],
                magic_mock.mock_calls,
            )

    def test_show_values(self):
        magic_mock = MagicMock(return_value="the_return")
        with patch("salt.modules.helm._exec_string_return", magic_mock):
            self.assertEqual("the_return", helm.show_values("chart"))
            self.assertEqual(
                [call(commands=["show", "values", "chart"], flags=None, kvflags=None)],
                magic_mock.mock_calls,
            )

    def test_status(self):
        magic_mock = MagicMock(return_value={"test": True})
        with patch("salt.modules.helm._exec_dict_return", magic_mock):
            self.assertEqual({"test": True}, helm.status("release"))
            self.assertEqual(
                [call(commands=["status", "release"], flags=None, kvflags=None)],
                magic_mock.mock_calls,
            )

    def test_template(self):
        magic_mock = MagicMock(return_value="the_return")
        with patch("salt.modules.helm._exec_string_return", magic_mock):
            self.assertEqual("the_return", helm.template("name", "chart"))
            self.assertEqual(
                [
                    call(
                        commands=["template", "name", "chart"], flags=None, kvflags=None
                    )
                ],
                magic_mock.mock_calls,
            )

    def test_test(self):
        magic_mock = MagicMock(return_value="the_return")
        with patch("salt.modules.helm._exec_string_return", magic_mock):
            self.assertEqual("the_return", helm.test("release"))
            self.assertEqual(
                [call(commands=["test", "release"], flags=None, kvflags=None)],
                magic_mock.mock_calls,
            )

    def test_uninstall(self):
        magic_mock = MagicMock(return_value=True)
        with patch("salt.modules.helm._exec_true_return", magic_mock):
            self.assertEqual(True, helm.uninstall("release"))
            self.assertEqual(
                [call(commands=["uninstall", "release"], flags=None, kvflags=None)],
                magic_mock.mock_calls,
            )

    def test_upgrade(self):
        magic_mock = MagicMock(return_value=True)
        with patch("salt.modules.helm._exec_true_return", magic_mock):
            self.assertEqual(True, helm.upgrade("release", "chart"))
            self.assertEqual(
                [
                    call(
                        commands=["upgrade", "release", "chart"],
                        flags=None,
                        kvflags=None,
                    )
                ],
                magic_mock.mock_calls,
            )

    def test_verify(self):
        magic_mock = MagicMock(return_value=True)
        with patch("salt.modules.helm._exec_true_return", magic_mock):
            self.assertEqual(True, helm.verify("path"))
            self.assertEqual(
                [call(commands=["verify", "path"], flags=None, kvflags=None)],
                magic_mock.mock_calls,
            )

    def test_version(self):
        magic_mock = MagicMock(return_value="the_return")
        with patch("salt.modules.helm._exec_string_return", magic_mock):
            self.assertEqual("the_return", helm.version())
            self.assertEqual(
                [call(commands=["version"], flags=None, kvflags=None)],
                magic_mock.mock_calls,
            )



"""
    :codeauthor: :email:`Anthony Shaw <anthonyshaw@apache.org>`
"""


import salt.modules.napalm_route as napalm_route
import tests.support.napalm as napalm_test_support
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


def mock_net_load(template, *args, **kwargs):
    raise ValueError("incorrect template {}".format(template))


class NapalmRouteModuleTestCase(TestCase, LoaderModuleMockMixin):
    def setup_loader_modules(self):
        patcher = patch(
            "salt.utils.napalm.get_device",
            MagicMock(return_value=napalm_test_support.MockNapalmDevice()),
        )
        patcher.start()
        self.addCleanup(patcher.stop)
        module_globals = {
            "__salt__": {
                "config.get": MagicMock(
                    return_value={"test": {"driver": "test", "key": "2orgk34kgk34g"}}
                ),
                "file.file_exists": napalm_test_support.true,
                "file.join": napalm_test_support.join,
                "file.get_managed": napalm_test_support.get_managed_file,
                "random.hash": napalm_test_support.random_hash,
                "net.load_template": mock_net_load,
            }
        }

        return {napalm_route: module_globals}

    def test_show(self):
        ret = napalm_route.show("1.2.3.4")
        assert ret["out"] == napalm_test_support.TEST_ROUTE.copy()



"""
Access Salt's elemental release code-names.

.. versionadded:: 3000

Salt's feature release schedule is based on the Periodic Table, as described
in the :ref:`Version Numbers <version-numbers>` documentation.

When a feature was added (or removed) in a specific release, it can be
difficult to build out future-proof functionality that is dependent on
a naming scheme that moves.

For example, a state syntax needs to change to support an option that will be
removed in the future, but there are many Minion versions in use across an
infrastructure. It would be handy to use some Jinja syntax to check for these
instances to perform one state syntax over another.

A simple example might be something like the following:

.. code-block:: jinja

    {# a boolean check #}
    {% set option_deprecated = salt['salt_version.less_than']("3001") %}

    {% if option_deprecated %}
      <use old syntax>
    {% else %}
      <use new syntax>
    {% endif %}

"""


import logging

import salt.utils.versions
import salt.version

log = logging.getLogger(__name__)

__virtualname__ = "salt_version"


def __virtual__():
    """
    Only work on POSIX-like systems
    """
    return __virtualname__


def get_release_number(name):
    """
    Returns the release number of a given release code name in a
    ``MAJOR.PATCH`` format.

    If the release name has not been given an assigned release number, the
    function returns a string. If the release cannot be found, it returns
    ``None``.

    name
        The release code name for which to find a release number.

    CLI Example:

    .. code-block:: bash

        salt '*' salt_version.get_release_number 'Oxygen'
    """
    name = name.lower()
    version_map = salt.version.SaltStackVersion.LNAMES
    version = version_map.get(name)
    if version is None:
        log.info("Version %s not found.", name)
        return None

    try:
        if version[1] == 0:
            log.info(
                "Version %s found, but no release number has been assigned yet.", name
            )
            return "No version assigned."
    except IndexError:
        # The new versioning scheme does not include minor version
        pass

    return ".".join(str(item) for item in version)


def equal(name):
    """
    Returns a boolean (True) if the minion's current version
    code name matches the named version.

    name
        The release code name to check the version against.

    CLI Example:

    .. code-block:: bash

        salt '*' salt_version.equal 'Oxygen'
    """
    if _check_release_cmp(name) == 0:
        log.info("The minion's version code name matches '%s'.", name)
        return True

    return False


def greater_than(name):
    """
    Returns a boolean (True) if the minion's current
    version code name is greater than the named version.

    name
        The release code name to check the version against.

    CLI Example:

    .. code-block:: bash

        salt '*' salt_version.greater_than 'Oxygen'
    """
    if _check_release_cmp(name) == 1:
        log.info("The minion's version code name is greater than '%s'.", name)
        return True

    return False


def less_than(name):
    """
    Returns a boolean (True) if the minion's current
    version code name is less than the named version.

    name
        The release code name to check the version against.

    CLI Example:

    .. code-block:: bash

        salt '*' salt_version.less_than 'Oxygen'
    """
    if _check_release_cmp(name) == -1:
        log.info("The minion's version code name is less than '%s'.", name)
        return True

    return False


def _check_release_cmp(name):
    """
    Helper function to compare the minion's current
    Salt version to release code name versions.

    If release code name isn't found, the function returns None. Otherwise, it
    returns the results of the version comparison as documented by the
    ``versions_cmp`` function in ``salt.utils.versions.py``.
    """
    map_version = get_release_number(name)
    if map_version is None:
        log.info("Release code name %s was not found.", name)
        return None

    current_version = str(salt.version.SaltStackVersion(*salt.version.__version_info__))
    current_version = current_version.rsplit(".", 1)[0]
    version_cmp = salt.utils.versions.version_cmp(current_version, map_version)
    return version_cmp



"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""
import pytest
import salt.states.layman as layman
from tests.support.mock import MagicMock, patch


@pytest.fixture
def configure_loader_modules():
    return {layman: {}}


def test_present():
    """
    Test to verify that the overlay is present.
    """
    name = "sunrise"

    ret = {"name": name, "result": True, "comment": "", "changes": {}}

    mock = MagicMock(side_effect=[[name], []])
    with patch.dict(layman.__salt__, {"layman.list_local": mock}):
        comt = "Overlay {} already present".format(name)
        ret.update({"comment": comt})
        assert layman.present(name) == ret

        with patch.dict(layman.__opts__, {"test": True}):
            comt = "Overlay {} is set to be added".format(name)
            ret.update({"comment": comt, "result": None})
            assert layman.present(name) == ret


def test_absent():
    """
    Test to verify that the overlay is absent.
    """
    name = "sunrise"

    ret = {"name": name, "result": True, "comment": "", "changes": {}}

    mock = MagicMock(side_effect=[[], [name]])
    with patch.dict(layman.__salt__, {"layman.list_local": mock}):
        comt = "Overlay {} already absent".format(name)
        ret.update({"comment": comt})
        assert layman.absent(name) == ret

        with patch.dict(layman.__opts__, {"test": True}):
            comt = "Overlay {} is set to be deleted".format(name)
            ret.update({"comment": comt, "result": None})
            assert layman.absent(name) == ret



"""
    :codeauthor: Rupesh Tare <rupesht@saltstack.com>
"""

import os
import textwrap

import salt.modules.dnsmasq as dnsmasq
from salt.exceptions import CommandExecutionError
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, mock_open, patch
from tests.support.unit import TestCase


class DnsmasqTestCase(TestCase, LoaderModuleMockMixin):
    """
    TestCase for the salt.modules.at module
    """

    def setup_loader_modules(self):
        return {dnsmasq: {}}

    def test_version(self):
        """
        test to show installed version of dnsmasq.
        """
        mock = MagicMock(return_value="A B C")
        with patch.dict(dnsmasq.__salt__, {"cmd.run": mock}):
            self.assertEqual(dnsmasq.version(), "C")

    def test_fullversion(self):
        """
        Test to Show installed version of dnsmasq and compile options.
        """
        mock = MagicMock(return_value="A B C\nD E F G H I")
        with patch.dict(dnsmasq.__salt__, {"cmd.run": mock}):
            self.assertDictEqual(
                dnsmasq.fullversion(),
                {"version": "C", "compile options": ["G", "H", "I"]},
            )

    def test_set_config(self):
        """
        test to show installed version of dnsmasq.
        """
        mock = MagicMock(return_value={"conf-dir": "A"})
        with patch.object(dnsmasq, "get_config", mock):
            mock = MagicMock(return_value=[".", "~", "bak", "#"])
            with patch.object(os, "listdir", mock):
                self.assertDictEqual(dnsmasq.set_config(), {})

    def test_set_config_filter_pub_kwargs(self):
        """
        Test that the kwargs returned from running the set_config function
        do not contain the __pub that may have been passed through in **kwargs.
        """
        with patch(
            "salt.modules.dnsmasq.get_config", MagicMock(return_value={"conf-dir": "A"})
        ):
            mock_domain = "local"
            mock_address = "/some-test-address.local/8.8.4.4"
            with patch.dict(dnsmasq.__salt__, {"file.append": MagicMock()}):
                ret = dnsmasq.set_config(
                    follow=False,
                    domain=mock_domain,
                    address=mock_address,
                    __pub_pid=8184,
                    __pub_jid=20161101194639387946,
                    __pub_tgt="salt-call",
                )
            self.assertEqual(ret, {"domain": mock_domain, "address": mock_address})

    def test_get_config(self):
        """
        test to dumps all options from the config file.
        """
        mock = MagicMock(return_value={"conf-dir": "A"})
        with patch.object(dnsmasq, "get_config", mock):
            mock = MagicMock(return_value=[".", "~", "bak", "#"])
            with patch.object(os, "listdir", mock):
                self.assertDictEqual(dnsmasq.get_config(), {"conf-dir": "A"})

    def test_parse_dnsmasq_no_file(self):
        """
        Tests that a CommandExecutionError is when a filename that doesn't exist is
        passed in.
        """
        self.assertRaises(CommandExecutionError, dnsmasq._parse_dnamasq, "filename")

    def test_parse_dnamasq(self):
        """
        test for generic function for parsing dnsmasq files including includes.
        """
        with patch("os.path.isfile", MagicMock(return_value=True)):
            text_file_data = textwrap.dedent(
                """\
                line here
                second line
                A=B
                #"""
            )
            with patch("salt.utils.files.fopen", mock_open(read_data=text_file_data)):
                self.assertDictEqual(
                    dnsmasq._parse_dnamasq("filename"),
                    {"A": "B", "unparsed": ["line here\n", "second line\n"]},
                )



# pylint: skip-file
from __future__ import absolute_import, division, print_function

import socket

from salt.ext.tornado.http1connection import HTTP1Connection
from salt.ext.tornado.httputil import HTTPMessageDelegate
from salt.ext.tornado.iostream import IOStream
from salt.ext.tornado.locks import Event
from salt.ext.tornado.netutil import add_accept_handler
from salt.ext.tornado.testing import AsyncTestCase, bind_unused_port, gen_test


class HTTP1ConnectionTest(AsyncTestCase):
    def setUp(self):
        super(HTTP1ConnectionTest, self).setUp()
        self.asyncSetUp()

    @gen_test
    def asyncSetUp(self):
        listener, port = bind_unused_port()
        event = Event()

        def accept_callback(conn, addr):
            self.server_stream = IOStream(conn)
            self.addCleanup(self.server_stream.close)
            event.set()

        add_accept_handler(listener, accept_callback)
        self.client_stream = IOStream(socket.socket())
        self.addCleanup(self.client_stream.close)
        yield [self.client_stream.connect(('127.0.0.1', port)),
               event.wait()]
        self.io_loop.remove_handler(listener)
        listener.close()

    @gen_test
    def test_http10_no_content_length(self):
        # Regression test for a bug in which can_keep_alive would crash
        # for an HTTP/1.0 (not 1.1) response with no content-length.
        conn = HTTP1Connection(self.client_stream, True)
        self.server_stream.write(b"HTTP/1.0 200 Not Modified\r\n\r\nhello")
        self.server_stream.close()

        event = Event()
        test = self
        body = []

        class Delegate(HTTPMessageDelegate):
            def headers_received(self, start_line, headers):
                test.code = start_line.code

            def data_received(self, data):
                body.append(data)

            def finish(self):
                event.set()

        yield conn.read_response(Delegate())
        yield event.wait()
        self.assertEqual(self.code, 200)
        self.assertEqual(b''.join(body), b'hello')



"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""


import salt.modules.introspect as introspect
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class IntrospectTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.modules.introspect
    """

    def setup_loader_modules(self):
        return {introspect: {}}

    # 'running_service_owners' function tests: 1

    def test_running_service_owners(self):
        """
        Test if it determine which packages own the currently running services.
        """
        err1 = (
            "The module for the package manager on this system does not"
            " support looking up which package(s) owns which file(s)"
        )
        err2 = (
            "The file module on this system does not "
            "support looking up open files on the system"
        )
        ret = {
            "Error": {
                "Unsupported File Module": "{}".format(err2),
                "Unsupported Package Manager": "{}".format(err1),
            }
        }
        self.assertDictEqual(introspect.running_service_owners(), ret)

        mock = MagicMock(return_value={})
        with patch.dict(
            introspect.__salt__,
            {"pkg.owner": mock, "file.open_files": mock, "service.execs": mock},
        ):
            self.assertDictEqual(introspect.running_service_owners(), {})

    # 'enabled_service_owners' function tests: 1

    def test_enabled_service_owners(self):
        """
        Test if it return which packages own each of the services
        that are currently enabled.
        """
        err1 = (
            "The module for the package manager on this system does not"
            " support looking up which package(s) owns which file(s)"
        )
        err2 = (
            "The module for the service manager on this system does not"
            " support showing descriptive service data"
        )
        ret = {
            "Error": {
                "Unsupported Service Manager": "{}".format(err2),
                "Unsupported Package Manager": "{}".format(err1),
            }
        }
        self.assertDictEqual(introspect.enabled_service_owners(), ret)

        mock = MagicMock(return_value={})
        with patch.dict(
            introspect.__salt__,
            {"pkg.owner": mock, "service.show": mock, "service.get_enabled": mock},
        ):
            self.assertDictEqual(introspect.enabled_service_owners(), {})

    # 'service_highstate' function tests: 1

    def test_service_highstate(self):
        """
        Test if it return running and enabled services in a highstate structure.
        """
        with patch(
            "salt.modules.introspect.running_service_owners", MagicMock(return_value={})
        ), patch(
            "salt.modules.introspect.enabled_service_owners", MagicMock(return_value={})
        ):
            self.assertDictEqual(introspect.service_highstate(), {})



"""
    :codeauthor: Rupesh Tare <rupesht@saltstack.com>
"""


import os.path

import salt.modules.devmap as devmap
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class DevMapTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.modules.devmap
    """

    def setup_loader_modules(self):
        return {devmap: {}}

    def test_multipath_list(self):
        """
        Test for Device-Mapper Multipath list
        """
        mock = MagicMock(return_value="A")
        with patch.dict(devmap.__salt__, {"cmd.run": mock}):
            self.assertEqual(devmap.multipath_list(), ["A"])

    def test_multipath_flush(self):
        """
        Test for Device-Mapper Multipath flush
        """
        mock = MagicMock(return_value=False)
        with patch.object(os.path, "exists", mock):
            self.assertEqual(devmap.multipath_flush("device"), "device does not exist")

        mock = MagicMock(return_value=True)
        with patch.object(os.path, "exists", mock):
            mock = MagicMock(return_value="A")
            with patch.dict(devmap.__salt__, {"cmd.run": mock}):
                self.assertEqual(devmap.multipath_flush("device"), ["A"])



"""
Manage CognitoIdentity Functions
================================

.. versionadded:: 2016.11.0

Create and destroy CognitoIdentity identity pools. Be aware that this interacts with
Amazon's services, and so may incur charges.

This module uses ``boto3``, which can be installed via package, or pip.

This module accepts explicit vpc credentials but can also utilize
IAM roles assigned to the instance through Instance Profiles. Dynamic
credentials are then automatically obtained from AWS API and no further
configuration is necessary. More information available `here
<http://docs.aws.amazon.com/AWSEC2/latest/UserGuide/iam-roles-for-amazon-ec2.html>`_.

If IAM roles are not used you need to specify them either in a pillar file or
in the minion's config file:

.. code-block:: yaml

    vpc.keyid: GKTADJGHEIQSXMKKRBJ08H
    vpc.key: askdjghsdfjkghWupUjasdflkdfklgjsdfjajkghs

It's also possible to specify ``key``, ``keyid`` and ``region`` via a profile,
either passed in as a dict, or as a string to pull from pillars or minion
config:

.. code-block:: yaml

    myprofile:
        keyid: GKTADJGHEIQSXMKKRBJ08H
        key: askdjghsdfjkghWupUjasdflkdfklgjsdfjajkghs
            region: us-east-1

.. code-block:: yaml

    Ensure function exists:
        boto_cognitoidentity.pool_present:
            - PoolName: my_identity_pool
            - region: us-east-1
            - keyid: GKTADJGHEIQSXMKKRBJ08H
            - key: askdjghsdfjkghWupUjasdflkdfklgjsdfjajkghs

"""


import logging

log = logging.getLogger(__name__)


def __virtual__():
    """
    Only load if boto is available.
    """
    if "boto_cognitoidentity.describe_identity_pools" in __salt__:
        return "boto_cognitoidentity"
    return (False, "boto_cognitoidentity module could not be loaded")


def _get_object(objname, objtype):
    """
    Helper function to retrieve objtype from pillars if objname
    is str, used for SupportedLoginProviders and
    OpenIdConnectProviderARNs.
    """
    ret = None
    if objname is None:
        return ret

    if isinstance(objname, str):
        if objname in __opts__:
            ret = __opts__[objname]
        master_opts = __pillar__.get("master", {})
        if objname in master_opts:
            ret = master_opts[objname]
        if objname in __pillar__:
            ret = __pillar__[objname]
    elif isinstance(objname, objtype):
        ret = objname

    if not isinstance(ret, objtype):
        ret = None

    return ret


def _role_present(
    ret, IdentityPoolId, AuthenticatedRole, UnauthenticatedRole, conn_params
):
    """
    Helper function to set the Roles to the identity pool
    """
    r = __salt__["boto_cognitoidentity.get_identity_pool_roles"](
        IdentityPoolName="", IdentityPoolId=IdentityPoolId, **conn_params
    )
    if r.get("error"):
        ret["result"] = False
        failure_comment = "Failed to get existing identity pool roles: {}".format(
            r["error"].get("message", r["error"])
        )
        ret["comment"] = "{}\n{}".format(ret["comment"], failure_comment)
        return

    existing_identity_pool_role = r.get("identity_pool_roles")[0].get("Roles", {})
    r = __salt__["boto_cognitoidentity.set_identity_pool_roles"](
        IdentityPoolId=IdentityPoolId,
        AuthenticatedRole=AuthenticatedRole,
        UnauthenticatedRole=UnauthenticatedRole,
        **conn_params
    )
    if not r.get("set"):
        ret["result"] = False
        failure_comment = "Failed to set roles: {}".format(
            r["error"].get("message", r["error"])
        )
        ret["comment"] = "{}\n{}".format(ret["comment"], failure_comment)
        return

    updated_identity_pool_role = r.get("roles")

    if existing_identity_pool_role != updated_identity_pool_role:
        if not ret["changes"]:
            ret["changes"]["old"] = dict()
            ret["changes"]["new"] = dict()
        ret["changes"]["old"]["Roles"] = existing_identity_pool_role
        ret["changes"]["new"]["Roles"] = r.get("roles")
        ret["comment"] = "{}\n{}".format(ret["comment"], "identity pool roles updated.")
    else:
        ret["comment"] = "{}\n{}".format(
            ret["comment"], "identity pool roles is already current."
        )

    return


def pool_present(
    name,
    IdentityPoolName,
    AuthenticatedRole,
    AllowUnauthenticatedIdentities=False,
    UnauthenticatedRole=None,
    SupportedLoginProviders=None,
    DeveloperProviderName=None,
    OpenIdConnectProviderARNs=None,
    region=None,
    key=None,
    keyid=None,
    profile=None,
):
    """
    Ensure Cognito Identity Pool exists.

    name
        The name of the state definition

    IdentityPoolName
        Name of the Cognito Identity Pool

    AuthenticatedRole
        An IAM role name or ARN that will be associated with temporary AWS
        credentials for an authenticated cognito identity.

    AllowUnauthenticatedIdentities
        Whether to allow anonymous user identities

    UnauthenticatedRole
        An IAM role name or ARN that will be associated with anonymous
        user identities

    SupportedLoginProviders
        A dictionary or pillar that contains key:value pairs mapping provider
        names to provider app IDs.

    DeveloperProviderName
        A string which is the domain by which Cognito will refer to your users.
        This name acts as a placeholder that allows your backend and the Cognito
        service to communicate about the developer provider.  Once you have set a
        developer provider name, you cannot change it.  Please take care in setting
        this parameter.

    OpenIdConnectProviderARNs
        A list or pillar name that contains a list of OpenID Connect provider ARNs.

    region
        Region to connect to.

    key
        Secret key to be used.

    keyid
        Access key to be used.

    profile
        A dict with region, key and keyid, or a pillar key (string) that
        contains a dict with region, key and keyid.
    """
    ret = {"name": IdentityPoolName, "result": True, "comment": "", "changes": {}}

    conn_params = dict(region=region, key=key, keyid=keyid, profile=profile)

    r = __salt__["boto_cognitoidentity.describe_identity_pools"](
        IdentityPoolName=IdentityPoolName, **conn_params
    )

    if r.get("error"):
        ret["result"] = False
        ret["comment"] = "Failed to describe identity pools {}".format(
            r["error"]["message"]
        )
        return ret

    identity_pools = r.get("identity_pools")
    if identity_pools and len(identity_pools) > 1:
        ret["result"] = False
        ret["comment"] = (
            "More than one identity pool for the given name matched "
            "Cannot execute pool_present function.\n"
            "Matched Identity Pools:\n{}".format(identity_pools)
        )
        return ret
    existing_identity_pool = None if identity_pools is None else identity_pools[0]
    IdentityPoolId = (
        None
        if existing_identity_pool is None
        else existing_identity_pool.get("IdentityPoolId")
    )

    if __opts__["test"]:
        if identity_pools is None:
            ret["comment"] = "A new identity pool named {} will be created.".format(
                IdentityPoolName
            )
        else:
            ret[
                "comment"
            ] = "An existing identity pool named {} with id {} will be updated.".format(
                IdentityPoolName, IdentityPoolId
            )
        ret["result"] = None
        return ret

    SupportedLoginProviders = _get_object(SupportedLoginProviders, dict)
    OpenIdConnectProviderARNs = _get_object(OpenIdConnectProviderARNs, list)

    request_params = dict(
        IdentityPoolName=IdentityPoolName,
        AllowUnauthenticatedIdentities=AllowUnauthenticatedIdentities,
        SupportedLoginProviders=SupportedLoginProviders,
        DeveloperProviderName=DeveloperProviderName,
        OpenIdConnectProviderARNs=OpenIdConnectProviderARNs,
    )
    request_params.update(conn_params)

    updated_identity_pool = None
    if IdentityPoolId is None:
        r = __salt__["boto_cognitoidentity.create_identity_pool"](**request_params)

        if r.get("created"):
            updated_identity_pool = r.get("identity_pool")
            IdentityPoolId = updated_identity_pool.get("IdentityPoolId")
            ret[
                "comment"
            ] = "A new identity pool with name {}, id {} is created.".format(
                IdentityPoolName, IdentityPoolId
            )
        else:
            ret["result"] = False
            ret["comment"] = "Failed to add a new identity pool: {}".format(
                r["error"].get("message", r["error"])
            )
            return ret
    else:  # Update an existing pool
        request_params["IdentityPoolId"] = IdentityPoolId
        # we will never change the IdentityPoolName from the state module
        request_params.pop("IdentityPoolName", None)
        r = __salt__["boto_cognitoidentity.update_identity_pool"](**request_params)

        if r.get("updated"):
            updated_identity_pool = r.get("identity_pool")
            ret[
                "comment"
            ] = "Existing identity pool with name {}, id {} is updated.".format(
                IdentityPoolName, IdentityPoolId
            )
        else:
            ret["result"] = False
            ret[
                "comment"
            ] = "Failed to update an existing identity pool {} {}: {}".format(
                IdentityPoolName,
                IdentityPoolId,
                r["error"].get("message", r["error"]),
            )
            return ret

    if existing_identity_pool != updated_identity_pool:
        ret["changes"]["old"] = dict()
        ret["changes"]["new"] = dict()
        change_key = "Identity Pool Name {}".format(IdentityPoolName)
        ret["changes"]["old"][change_key] = existing_identity_pool
        ret["changes"]["new"][change_key] = updated_identity_pool
    else:
        ret["comment"] = "Identity Pool state is current, no changes."

    # Now update the Auth/Unauth Roles
    _role_present(
        ret, IdentityPoolId, AuthenticatedRole, UnauthenticatedRole, conn_params
    )

    return ret


def pool_absent(
    name,
    IdentityPoolName,
    RemoveAllMatched=False,
    region=None,
    key=None,
    keyid=None,
    profile=None,
):
    """
    Ensure cognito identity pool with passed properties is absent.

    name
        The name of the state definition.

    IdentityPoolName
        Name of the Cognito Identity Pool.  Please note that this may
        match multiple pools with the same given name, in which case,
        all will be removed.

    RemoveAllMatched
        If True, all identity pools with the matching IdentityPoolName
        will be removed.  If False and there are more than one identity pool
        with the matching IdentityPoolName, no action will be taken.  If False
        and there is only one identity pool with the matching IdentityPoolName,
        the identity pool will be removed.

    region
        Region to connect to.

    key
        Secret key to be used.

    keyid
        Access key to be used.

    profile
        A dict with region, key and keyid, or a pillar key (string) that
        contains a dict with region, key and keyid.
    """

    ret = {"name": IdentityPoolName, "result": True, "comment": "", "changes": {}}

    conn_params = dict(region=region, key=key, keyid=keyid, profile=profile)

    r = __salt__["boto_cognitoidentity.describe_identity_pools"](
        IdentityPoolName=IdentityPoolName, **conn_params
    )

    if r.get("error"):
        ret["result"] = False
        ret["comment"] = "Failed to describe identity pools {}".format(
            r["error"]["message"]
        )
        return ret

    identity_pools = r.get("identity_pools")

    if identity_pools is None:
        ret["result"] = True
        ret["comment"] = "No matching identity pool for the given name {}".format(
            IdentityPoolName
        )
        return ret

    if not RemoveAllMatched and len(identity_pools) > 1:
        ret["result"] = False
        ret["comment"] = (
            "More than one identity pool for the given name matched "
            "and RemoveAllMatched flag is False.\n"
            "Matched Identity Pools:\n{}".format(identity_pools)
        )
        return ret

    if __opts__["test"]:
        ret[
            "comment"
        ] = "The following matched identity pools will be deleted.\n{}".format(
            identity_pools
        )
        ret["result"] = None
        return ret

    for identity_pool in identity_pools:
        IdentityPoolId = identity_pool.get("IdentityPoolId")
        r = __salt__["boto_cognitoidentity.delete_identity_pools"](
            IdentityPoolName="", IdentityPoolId=IdentityPoolId, **conn_params
        )
        if r.get("error"):
            ret["result"] = False
            failure_comment = "Failed to delete identity pool {}: {}".format(
                IdentityPoolId, r["error"].get("message", r["error"])
            )
            ret["comment"] = "{}\n{}".format(ret["comment"], failure_comment)
            return ret

        if r.get("deleted"):
            if not ret["changes"]:
                ret["changes"]["old"] = dict()
                ret["changes"]["new"] = dict()
            change_key = "Identity Pool Id {}".format(IdentityPoolId)
            ret["changes"]["old"][change_key] = IdentityPoolName
            ret["changes"]["new"][change_key] = None
            ret["comment"] = "{}\n{}".format(
                ret["comment"], "{} deleted".format(change_key)
            )
        else:
            ret["result"] = False
            failure_comment = (
                "Identity Pool Id {} not deleted, returned count 0".format(
                    IdentityPoolId
                )
            )
            ret["comment"] = "{}\n{}".format(ret["comment"], failure_comment)
            return ret

    return ret



"""
Make me some salt!
"""


import os
import warnings

# We import log ASAP because we NEED to make sure that any logger instance salt
# instantiates is using salt.log.setup.SaltLoggingClass
import salt.log.setup
import salt.utils.kinds as kinds
from salt.exceptions import SaltClientError, SaltSystemExit, get_error_message

# the try block below bypasses an issue at build time so that modules don't
# cause the build to fail
from salt.utils import migrations
from salt.utils.platform import is_junos
from salt.utils.process import HAS_PSUTIL
from salt.utils.verify import verify_log

# All salt related deprecation warnings should be shown once each!
warnings.filterwarnings(
    "once",  # Show once
    "",  # No deprecation message matchHAS_PSUTIL
    DeprecationWarning,  # This filter is for DeprecationWarnings
    r"^(salt|salt\.(.*))$",  # Match module(s) 'salt' and 'salt.<whatever>'
    append=True,
)

# While we are supporting Python2.6, hide nested with-statements warnings
warnings.filterwarnings(
    "ignore",
    "With-statements now directly support multiple context managers",
    DeprecationWarning,
    append=True,
)

# Filter the backports package UserWarning about being re-imported
warnings.filterwarnings(
    "ignore",
    "^Module backports was already imported from (.*), but (.*) is being added to"
    " sys.path$",
    UserWarning,
    append=True,
)


try:
    import salt.utils.parsers
    from salt.utils.verify import check_user, verify_env, verify_socket
    from salt.utils.zeromq import ip_bracket
except ImportError as exc:
    if exc.args[0] != "No module named _msgpack":
        raise


# Let's instantiate log using salt.log.setup.logging.getLogger() so pylint
# leaves us alone and stops complaining about an un-used import
log = salt.log.setup.logging.getLogger(__name__)


class DaemonsMixin:  # pylint: disable=no-init
    """
    Uses the same functions for all daemons
    """

    def verify_hash_type(self):
        """
        Verify and display a nag-messsage to the log if vulnerable hash-type is used.

        :return:
        """
        if self.config["hash_type"].lower() in ["md5", "sha1"]:
            log.warning(
                "IMPORTANT: Do not use %s hashing algorithm! Please set "
                '"hash_type" to sha256 in Salt %s config!',
                self.config["hash_type"],
                self.__class__.__name__,
            )

    def action_log_info(self, action):
        """
        Say daemon starting.

        :param action
        :return:
        """
        log.info("%s the Salt %s", action, self.__class__.__name__)

    def start_log_info(self):
        """
        Say daemon starting.

        :return:
        """
        log.info("The Salt %s is starting up", self.__class__.__name__)

    def shutdown_log_info(self):
        """
        Say daemon shutting down.

        :return:
        """
        log.info("The Salt %s is shut down", self.__class__.__name__)

    def environment_failure(self, error):
        """
        Log environment failure for the daemon and exit with the error code.

        :param error:
        :return:
        """
        log.exception(
            "Failed to create environment for %s: %s",
            self.__class__.__name__,
            get_error_message(error),
        )
        self.shutdown(error)


class Master(
    salt.utils.parsers.MasterOptionParser, DaemonsMixin
):  # pylint: disable=no-init
    """
    Creates a master server
    """

    def _handle_signals(self, signum, sigframe):
        if hasattr(self.master, "process_manager"):
            # escalate signal to the process manager processes
            self.master.process_manager._handle_signals(signum, sigframe)
        super()._handle_signals(signum, sigframe)

    def prepare(self):
        """
        Run the preparation sequence required to start a salt master server.

        If sub-classed, don't **ever** forget to run:

            super(YourSubClass, self).prepare()
        """
        super().prepare()

        try:
            if self.config["verify_env"]:
                v_dirs = [
                    self.config["pki_dir"],
                    os.path.join(self.config["pki_dir"], "minions"),
                    os.path.join(self.config["pki_dir"], "minions_pre"),
                    os.path.join(self.config["pki_dir"], "minions_denied"),
                    os.path.join(self.config["pki_dir"], "minions_autosign"),
                    os.path.join(self.config["pki_dir"], "minions_rejected"),
                    self.config["cachedir"],
                    os.path.join(self.config["cachedir"], "jobs"),
                    os.path.join(self.config["cachedir"], "proc"),
                    self.config["sock_dir"],
                    self.config["token_dir"],
                    self.config["syndic_dir"],
                    self.config["sqlite_queue_dir"],
                ]
                verify_env(
                    v_dirs,
                    self.config["user"],
                    permissive=self.config["permissive_pki_access"],
                    root_dir=self.config["root_dir"],
                    pki_dir=self.config["pki_dir"],
                )
                # Clear out syndics from cachedir
                for syndic_file in os.listdir(self.config["syndic_dir"]):
                    os.remove(os.path.join(self.config["syndic_dir"], syndic_file))
        except OSError as error:
            self.environment_failure(error)

        self.setup_logfile_logger()
        verify_log(self.config)
        self.action_log_info("Setting up")

        # TODO: AIO core is separate from transport
        if not verify_socket(
            self.config["interface"],
            self.config["publish_port"],
            self.config["ret_port"],
        ):
            self.shutdown(4, "The ports are not available to bind")
        self.config["interface"] = ip_bracket(self.config["interface"])
        migrations.migrate_paths(self.config)

        # Late import so logging works correctly
        import salt.master

        self.master = salt.master.Master(self.config)

        self.daemonize_if_required()
        self.set_pidfile()
        salt.utils.process.notify_systemd()

    def start(self):
        """
        Start the actual master.

        If sub-classed, don't **ever** forget to run:

            super(YourSubClass, self).start()

        NOTE: Run any required code before calling `super()`.
        """
        super().start()
        if check_user(self.config["user"]):
            self.action_log_info("Starting up")
            self.verify_hash_type()
            self.master.start()

    def shutdown(self, exitcode=0, exitmsg=None):
        """
        If sub-classed, run any shutdown operations on this method.
        """
        self.shutdown_log_info()
        msg = "The salt master is shutdown. "
        if exitmsg is not None:
            exitmsg = msg + exitmsg
        else:
            exitmsg = msg.strip()
        super().shutdown(exitcode, exitmsg)


class Minion(
    salt.utils.parsers.MinionOptionParser, DaemonsMixin
):  # pylint: disable=no-init
    """
    Create a minion server
    """

    def _handle_signals(self, signum, sigframe):  # pylint: disable=unused-argument
        # escalate signal to the process manager processes
        if hasattr(self.minion, "stop"):
            self.minion.stop(signum)
        super()._handle_signals(signum, sigframe)

    # pylint: disable=no-member
    def prepare(self):
        """
        Run the preparation sequence required to start a salt minion.

        If sub-classed, don't **ever** forget to run:

            super(YourSubClass, self).prepare()
        """
        super().prepare()

        try:
            if self.config["verify_env"]:
                confd = self.config.get("default_include")
                if confd:
                    # If 'default_include' is specified in config, then use it
                    if "*" in confd:
                        # Value is of the form "minion.d/*.conf"
                        confd = os.path.dirname(confd)
                    if not os.path.isabs(confd):
                        # If configured 'default_include' is not an absolute
                        # path, consider it relative to folder of 'conf_file'
                        # (/etc/salt by default)
                        confd = os.path.join(
                            os.path.dirname(self.config["conf_file"]), confd
                        )
                else:
                    confd = os.path.join(
                        os.path.dirname(self.config["conf_file"]), "minion.d"
                    )

                v_dirs = [
                    self.config["pki_dir"],
                    self.config["cachedir"],
                    self.config["sock_dir"],
                    self.config["extension_modules"],
                    confd,
                ]

                verify_env(
                    v_dirs,
                    self.config["user"],
                    permissive=self.config["permissive_pki_access"],
                    root_dir=self.config["root_dir"],
                    pki_dir=self.config["pki_dir"],
                )
        except OSError as error:
            self.environment_failure(error)

        self.setup_logfile_logger()
        verify_log(self.config)
        log.info('Setting up the Salt Minion "%s"', self.config["id"])
        migrations.migrate_paths(self.config)

        # Bail out if we find a process running and it matches out pidfile
        if (HAS_PSUTIL and not self.claim_process_responsibility()) or (
            not HAS_PSUTIL and self.check_running()
        ):
            self.action_log_info("An instance is already running. Exiting")
            self.shutdown(1)

        transport = self.config.get("transport").lower()

        # TODO: AIO core is separate from transport
        if transport in ("zeromq", "tcp", "detect"):
            # Late import so logging works correctly
            import salt.minion

            # If the minion key has not been accepted, then Salt enters a loop
            # waiting for it, if we daemonize later then the minion could halt
            # the boot process waiting for a key to be accepted on the master.
            # This is the latest safe place to daemonize
            self.daemonize_if_required()
            self.set_pidfile()
            if self.config.get("master_type") == "func":
                salt.minion.eval_master_func(self.config)
            self.minion = salt.minion.MinionManager(self.config)
        else:
            log.error(
                "The transport '%s' is not supported. Please use one of "
                "the following: tcp, zeromq, or detect.",
                transport,
            )
            self.shutdown(1)

    def start(self):
        """
        Start the actual minion.

        If sub-classed, don't **ever** forget to run:

            super(YourSubClass, self).start()

        NOTE: Run any required code before calling `super()`.
        """
        super().start()
        while True:
            try:
                self._real_start()
            except SaltClientError as exc:
                # Restart for multi_master failover when daemonized
                if self.options.daemon:
                    continue
            break

    def _real_start(self):
        try:
            if check_user(self.config["user"]):
                self.action_log_info("Starting up")
                self.verify_hash_type()
                self.minion.tune_in()
                if self.minion.restart:
                    raise SaltClientError("Minion could not connect to Master")
        except (KeyboardInterrupt, SaltSystemExit) as error:
            self.action_log_info("Stopping")
            if isinstance(error, KeyboardInterrupt):
                log.warning("Exiting on Ctrl-c")
                self.shutdown()
            else:
                log.error(error)
                self.shutdown(error.code)

    def call(self, cleanup_protecteds):
        """
        Start the actual minion as a caller minion.

        cleanup_protecteds is list of yard host addresses that should not be
        cleaned up this is to fix race condition when salt-caller minion starts up

        If sub-classed, don't **ever** forget to run:

            super(YourSubClass, self).start()

        NOTE: Run any required code before calling `super()`.
        """
        try:
            self.prepare()
            if check_user(self.config["user"]):
                self.minion.opts["__role"] = kinds.APPL_KIND_NAMES[
                    kinds.applKinds.caller
                ]
                self.minion.call_in()
        except (KeyboardInterrupt, SaltSystemExit) as exc:
            self.action_log_info("Stopping")
            if isinstance(exc, KeyboardInterrupt):
                log.warning("Exiting on Ctrl-c")
                self.shutdown()
            else:
                log.error(exc)
                self.shutdown(exc.code)

    def shutdown(self, exitcode=0, exitmsg=None):
        """
        If sub-classed, run any shutdown operations on this method.

        :param exitcode
        :param exitmsg
        """
        self.action_log_info("Shutting down")
        if hasattr(self, "minion") and hasattr(self.minion, "destroy"):
            self.minion.destroy()
        super().shutdown(
            exitcode,
            (
                "The Salt {} is shutdown. {}".format(
                    self.__class__.__name__, (exitmsg or "")
                ).strip()
            ),
        )

    # pylint: enable=no-member


class ProxyMinion(
    salt.utils.parsers.ProxyMinionOptionParser, DaemonsMixin
):  # pylint: disable=no-init
    """
    Create a proxy minion server
    """

    def _handle_signals(self, signum, sigframe):  # pylint: disable=unused-argument
        # escalate signal to the process manager processes
        self.minion.stop(signum)
        super()._handle_signals(signum, sigframe)

    # pylint: disable=no-member
    def prepare(self):
        """
        Run the preparation sequence required to start a salt proxy minion.

        If sub-classed, don't **ever** forget to run:

            super(YourSubClass, self).prepare()
        """
        super().prepare()

        ## allow for native minion
        if not is_junos():
            if not self.values.proxyid:
                self.error("salt-proxy requires --proxyid")

        # Proxies get their ID from the command line.  This may need to change in
        # the future.
        # We used to set this here.  Now it is set in ProxyMinionOptionParser
        # by passing it via setup_config to config.minion_config
        # self.config['id'] = self.values.proxyid

        try:
            if self.config["verify_env"]:
                confd = self.config.get("default_include")
                if confd:
                    # If 'default_include' is specified in config, then use it
                    if "*" in confd:
                        # Value is of the form "minion.d/*.conf"
                        confd = os.path.dirname(confd)
                    if not os.path.isabs(confd):
                        # If configured 'default_include' is not an absolute
                        # path, consider it relative to folder of 'conf_file'
                        # (/etc/salt by default)
                        confd = os.path.join(
                            os.path.dirname(self.config["conf_file"]), confd
                        )
                else:
                    confd = os.path.join(
                        os.path.dirname(self.config["conf_file"]), "proxy.d"
                    )

                v_dirs = [
                    self.config["pki_dir"],
                    self.config["cachedir"],
                    self.config["sock_dir"],
                    self.config["extension_modules"],
                    confd,
                ]

                verify_env(
                    v_dirs,
                    self.config["user"],
                    permissive=self.config["permissive_pki_access"],
                    root_dir=self.config["root_dir"],
                    pki_dir=self.config["pki_dir"],
                )
        except OSError as error:
            self.environment_failure(error)

        self.setup_logfile_logger()
        verify_log(self.config)
        self.action_log_info('Setting up "{}"'.format(self.config["id"]))

        migrations.migrate_paths(self.config)

        # Bail out if we find a process running and it matches out pidfile
        if self.check_running():
            self.action_log_info("An instance is already running. Exiting")
            self.shutdown(1)

        # TODO: AIO core is separate from transport
        # Late import so logging works correctly
        import salt.minion

        # If the minion key has not been accepted, then Salt enters a loop
        # waiting for it, if we daemonize later then the minion could halt
        # the boot process waiting for a key to be accepted on the master.
        # This is the latest safe place to daemonize
        self.daemonize_if_required()
        self.set_pidfile()
        if self.config.get("master_type") == "func":
            salt.minion.eval_master_func(self.config)
        self.minion = salt.minion.ProxyMinionManager(self.config)

    def start(self):
        """
        Start the actual proxy minion.

        If sub-classed, don't **ever** forget to run:

            super(YourSubClass, self).start()

        NOTE: Run any required code before calling `super()`.
        """
        super().start()
        try:
            if check_user(self.config["user"]):
                self.action_log_info("The Proxy Minion is starting up")
                self.verify_hash_type()
                self.minion.tune_in()
                if self.minion.restart:
                    raise SaltClientError("Proxy Minion could not connect to Master")
        except (KeyboardInterrupt, SaltSystemExit) as exc:
            self.action_log_info("Proxy Minion Stopping")
            if isinstance(exc, KeyboardInterrupt):
                log.warning("Exiting on Ctrl-c")
                self.shutdown()
            else:
                log.error(exc)
                self.shutdown(exc.code)

    def shutdown(self, exitcode=0, exitmsg=None):
        """
        If sub-classed, run any shutdown operations on this method.

        :param exitcode
        :param exitmsg
        """
        if hasattr(self, "minion") and "proxymodule" in self.minion.opts:
            proxy_fn = self.minion.opts["proxymodule"].loaded_base_name + ".shutdown"
            self.minion.opts["proxymodule"][proxy_fn](self.minion.opts)
        self.action_log_info("Shutting down")
        super().shutdown(
            exitcode,
            (
                "The Salt {} is shutdown. {}".format(
                    self.__class__.__name__, (exitmsg or "")
                ).strip()
            ),
        )

    # pylint: enable=no-member


class Syndic(
    salt.utils.parsers.SyndicOptionParser, DaemonsMixin
):  # pylint: disable=no-init
    """
    Create a syndic server
    """

    def prepare(self):
        """
        Run the preparation sequence required to start a salt syndic minion.

        If sub-classed, don't **ever** forget to run:

            super(YourSubClass, self).prepare()
        """
        super().prepare()
        try:
            if self.config["verify_env"]:
                verify_env(
                    [
                        self.config["pki_dir"],
                        self.config["cachedir"],
                        self.config["sock_dir"],
                        self.config["extension_modules"],
                    ],
                    self.config["user"],
                    permissive=self.config["permissive_pki_access"],
                    root_dir=self.config["root_dir"],
                    pki_dir=self.config["pki_dir"],
                )
        except OSError as error:
            self.environment_failure(error)

        self.setup_logfile_logger()
        verify_log(self.config)
        self.action_log_info('Setting up "{}"'.format(self.config["id"]))

        # Late import so logging works correctly
        import salt.minion

        self.daemonize_if_required()
        self.syndic = salt.minion.SyndicManager(self.config)
        self.set_pidfile()

    def start(self):
        """
        Start the actual syndic.

        If sub-classed, don't **ever** forget to run:

            super(YourSubClass, self).start()

        NOTE: Run any required code before calling `super()`.
        """
        super().start()
        if check_user(self.config["user"]):
            self.action_log_info("Starting up")
            self.verify_hash_type()
            try:
                self.syndic.tune_in()
            except KeyboardInterrupt:
                self.action_log_info("Stopping")
                self.shutdown()

    def shutdown(self, exitcode=0, exitmsg=None):
        """
        If sub-classed, run any shutdown operations on this method.

        :param exitcode
        :param exitmsg
        """
        self.action_log_info("Shutting down")
        super().shutdown(
            exitcode,
            (
                "The Salt {} is shutdown. {}".format(
                    self.__class__.__name__, (exitmsg or "")
                ).strip()
            ),
        )



"""
    :codeauthor: :email:`Roald Nefs (info@roaldnefs.com)`

    tests.unit.returners.telegram_return_test
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
"""


import salt.returners.telegram_return as telegram
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class TelegramReturnerTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test Telegram Returner
    """

    def setup_loader_modules(self):
        return {telegram: {}}

    def test_returner(self):
        """
        Test to see if the Telegram returner sends a message
        """
        ret = {
            "id": "12345",
            "fun": "mytest.func",
            "fun_args": "myfunc args",
            "jid": "54321",
            "return": "The room is on fire as shes fixing her hair",
        }
        options = {"chat_id": "", "token": ""}

        with patch(
            "salt.returners.telegram_return._get_options",
            MagicMock(return_value=options),
        ), patch.dict(
            "salt.returners.telegram_return.__salt__",
            {"telegram.post_message": MagicMock(return_value=True)},
        ):
            self.assertTrue(telegram.returner(ret))



import copy
import logging
import random
import string

import salt.loader
import salt.modules.boto_elasticsearch_domain as boto_elasticsearch_domain
from salt.utils.versions import LooseVersion
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase, skipIf

# pylint: disable=import-error,no-name-in-module
try:
    import boto3
    from botocore.exceptions import ClientError

    HAS_BOTO = True
except ImportError:
    HAS_BOTO = False


# pylint: enable=import-error,no-name-in-module

# the boto_elasticsearch_domain module relies on the connect_to_region() method
# which was added in boto 2.8.0
# https://github.com/boto/boto/commit/33ac26b416fbb48a60602542b4ce15dcc7029f12
required_boto3_version = "1.2.1"


def _has_required_boto():
    """
    Returns True/False boolean depending on if Boto is installed and correct
    version.
    """
    if not HAS_BOTO:
        return False
    elif LooseVersion(boto3.__version__) < LooseVersion(required_boto3_version):
        return False
    else:
        return True


if _has_required_boto():
    region = "us-east-1"
    access_key = "GKTADJGHEIQSXMKKRBJ08H"
    secret_key = "askdjghsdfjkghWupUjasdflkdfklgjsdfjajkghs"
    conn_parameters = {
        "region": region,
        "key": access_key,
        "keyid": secret_key,
        "profile": {},
    }
    error_message = (
        "An error occurred (101) when calling the {0} operation: Test-defined error"
    )
    error_content = {"Error": {"Code": 101, "Message": "Test-defined error"}}
    not_found_error = ClientError(
        {
            "Error": {
                "Code": "ResourceNotFoundException",
                "Message": "Test-defined error",
            }
        },
        "msg",
    )
    domain_ret = dict(
        DomainName="testdomain",
        ElasticsearchClusterConfig={},
        EBSOptions={},
        AccessPolicies={},
        SnapshotOptions={},
        AdvancedOptions={},
    )

log = logging.getLogger(__name__)


class BotoElasticsearchDomainTestCaseBase(TestCase, LoaderModuleMockMixin):
    conn = None

    def setup_loader_modules(self):
        self.opts = salt.config.DEFAULT_MINION_OPTS.copy()
        utils = salt.loader.utils(
            self.opts,
            whitelist=["boto3", "args", "systemd", "path", "platform"],
            context={},
        )
        return {boto_elasticsearch_domain: {"__utils__": utils}}

    def setUp(self):
        super().setUp()
        boto_elasticsearch_domain.__init__(self.opts)
        del self.opts

        # Set up MagicMock to replace the boto3 session
        # connections keep getting cached from prior tests, can't find the
        # correct context object to clear it. So randomize the cache key, to prevent any
        # cache hits
        conn_parameters["key"] = "".join(
            random.choice(string.ascii_lowercase + string.digits) for _ in range(50)
        )

        self.patcher = patch("boto3.session.Session")
        self.addCleanup(self.patcher.stop)
        self.addCleanup(delattr, self, "patcher")
        mock_session = self.patcher.start()

        session_instance = mock_session.return_value
        self.conn = MagicMock()
        self.addCleanup(delattr, self, "conn")
        session_instance.client.return_value = self.conn


class BotoElasticsearchDomainTestCaseMixin:
    pass


# @skipIf(True, "Skip these tests while investigating failures")
@skipIf(HAS_BOTO is False, "The boto module must be installed.")
@skipIf(
    _has_required_boto() is False,
    "The boto3 module must be greater than or equal to version {}".format(
        required_boto3_version
    ),
)
class BotoElasticsearchDomainTestCase(
    BotoElasticsearchDomainTestCaseBase, BotoElasticsearchDomainTestCaseMixin
):
    """
    TestCase for salt.modules.boto_elasticsearch_domain module
    """

    def test_that_when_checking_if_a_domain_exists_and_a_domain_exists_the_domain_exists_method_returns_true(
        self,
    ):
        """
        Tests checking domain existence when the domain already exists
        """
        result = boto_elasticsearch_domain.exists(
            DomainName="testdomain", **conn_parameters
        )

        self.assertTrue(result["exists"])

    def test_that_when_checking_if_a_domain_exists_and_a_domain_does_not_exist_the_domain_exists_method_returns_false(
        self,
    ):
        """
        Tests checking domain existence when the domain does not exist
        """
        self.conn.describe_elasticsearch_domain.side_effect = not_found_error
        result = boto_elasticsearch_domain.exists(
            DomainName="mydomain", **conn_parameters
        )

        self.assertFalse(result["exists"])

    def test_that_when_checking_if_a_domain_exists_and_boto3_returns_an_error_the_domain_exists_method_returns_error(
        self,
    ):
        """
        Tests checking domain existence when boto returns an error
        """
        self.conn.describe_elasticsearch_domain.side_effect = ClientError(
            error_content, "list_domains"
        )
        result = boto_elasticsearch_domain.exists(
            DomainName="mydomain", **conn_parameters
        )

        self.assertEqual(
            result.get("error", {}).get("message"), error_message.format("list_domains")
        )

    def test_that_when_checking_domain_status_and_a_domain_exists_the_domain_status_method_returns_info(
        self,
    ):
        """
        Tests checking domain existence when the domain already exists
        """
        self.conn.describe_elasticsearch_domain.return_value = {
            "DomainStatus": domain_ret
        }
        result = boto_elasticsearch_domain.status(
            DomainName="testdomain", **conn_parameters
        )

        self.assertTrue(result["domain"])

    def test_that_when_checking_domain_status_and_boto3_returns_an_error_the_domain_status_method_returns_error(
        self,
    ):
        """
        Tests checking domain existence when boto returns an error
        """
        self.conn.describe_elasticsearch_domain.side_effect = ClientError(
            error_content, "list_domains"
        )
        result = boto_elasticsearch_domain.status(
            DomainName="mydomain", **conn_parameters
        )

        self.assertEqual(
            result.get("error", {}).get("message"), error_message.format("list_domains")
        )

    def test_that_when_describing_domain_it_returns_the_dict_of_properties_returns_true(
        self,
    ):
        """
        Tests describing parameters if domain exists
        """
        domainconfig = {}
        for k, v in domain_ret.items():
            if k == "DomainName":
                continue
            domainconfig[k] = {"Options": v}
        self.conn.describe_elasticsearch_domain_config.return_value = {
            "DomainConfig": domainconfig
        }

        result = boto_elasticsearch_domain.describe(
            DomainName=domain_ret["DomainName"], **conn_parameters
        )

        log.warning(result)
        desired_ret = copy.copy(domain_ret)
        desired_ret.pop("DomainName")
        self.assertEqual(result, {"domain": desired_ret})

    def test_that_when_describing_domain_on_client_error_it_returns_error(self):
        """
        Tests describing parameters failure
        """
        self.conn.describe_elasticsearch_domain_config.side_effect = ClientError(
            error_content, "list_domains"
        )
        result = boto_elasticsearch_domain.describe(
            DomainName="testdomain", **conn_parameters
        )
        self.assertTrue("error" in result)

    def test_that_when_creating_a_domain_succeeds_the_create_domain_method_returns_true(
        self,
    ):
        """
        tests True domain created.
        """
        self.conn.create_elasticsearch_domain.return_value = {
            "DomainStatus": domain_ret
        }
        args = copy.copy(domain_ret)
        args.update(conn_parameters)
        result = boto_elasticsearch_domain.create(**args)

        self.assertTrue(result["created"])

    def test_that_when_creating_a_domain_fails_the_create_domain_method_returns_error(
        self,
    ):
        """
        tests False domain not created.
        """
        self.conn.create_elasticsearch_domain.side_effect = ClientError(
            error_content, "create_domain"
        )
        args = copy.copy(domain_ret)
        args.update(conn_parameters)
        result = boto_elasticsearch_domain.create(**args)
        self.assertEqual(
            result.get("error", {}).get("message"),
            error_message.format("create_domain"),
        )

    def test_that_when_deleting_a_domain_succeeds_the_delete_domain_method_returns_true(
        self,
    ):
        """
        tests True domain deleted.
        """
        result = boto_elasticsearch_domain.delete(
            DomainName="testdomain", **conn_parameters
        )
        self.assertTrue(result["deleted"])

    def test_that_when_deleting_a_domain_fails_the_delete_domain_method_returns_false(
        self,
    ):
        """
        tests False domain not deleted.
        """
        self.conn.delete_elasticsearch_domain.side_effect = ClientError(
            error_content, "delete_domain"
        )
        result = boto_elasticsearch_domain.delete(
            DomainName="testdomain", **conn_parameters
        )
        self.assertFalse(result["deleted"])

    def test_that_when_updating_a_domain_succeeds_the_update_domain_method_returns_true(
        self,
    ):
        """
        tests True domain updated.
        """
        self.conn.update_elasticsearch_domain_config.return_value = {
            "DomainConfig": domain_ret
        }
        args = copy.copy(domain_ret)
        args.update(conn_parameters)
        result = boto_elasticsearch_domain.update(**args)

        self.assertTrue(result["updated"])

    def test_that_when_updating_a_domain_fails_the_update_domain_method_returns_error(
        self,
    ):
        """
        tests False domain not updated.
        """
        self.conn.update_elasticsearch_domain_config.side_effect = ClientError(
            error_content, "update_domain"
        )
        args = copy.copy(domain_ret)
        args.update(conn_parameters)
        result = boto_elasticsearch_domain.update(**args)
        self.assertEqual(
            result.get("error", {}).get("message"),
            error_message.format("update_domain"),
        )

    def test_that_when_adding_tags_succeeds_the_add_tags_method_returns_true(self):
        """
        tests True tags added.
        """
        self.conn.describe_elasticsearch_domain.return_value = {
            "DomainStatus": domain_ret
        }
        result = boto_elasticsearch_domain.add_tags(
            DomainName="testdomain", a="b", **conn_parameters
        )

        self.assertTrue(result["tagged"])

    def test_that_when_adding_tags_fails_the_add_tags_method_returns_false(self):
        """
        tests False tags not added.
        """
        self.conn.add_tags.side_effect = ClientError(error_content, "add_tags")
        self.conn.describe_elasticsearch_domain.return_value = {
            "DomainStatus": domain_ret
        }
        result = boto_elasticsearch_domain.add_tags(
            DomainName=domain_ret["DomainName"], a="b", **conn_parameters
        )
        self.assertFalse(result["tagged"])

    def test_that_when_removing_tags_succeeds_the_remove_tags_method_returns_true(self):
        """
        tests True tags removed.
        """
        self.conn.describe_elasticsearch_domain.return_value = {
            "DomainStatus": domain_ret
        }
        result = boto_elasticsearch_domain.remove_tags(
            DomainName=domain_ret["DomainName"], TagKeys=["a"], **conn_parameters
        )

        self.assertTrue(result["tagged"])

    def test_that_when_removing_tags_fails_the_remove_tags_method_returns_false(self):
        """
        tests False tags not removed.
        """
        self.conn.remove_tags.side_effect = ClientError(error_content, "remove_tags")
        self.conn.describe_elasticsearch_domain.return_value = {
            "DomainStatus": domain_ret
        }
        result = boto_elasticsearch_domain.remove_tags(
            DomainName=domain_ret["DomainName"], TagKeys=["b"], **conn_parameters
        )
        self.assertFalse(result["tagged"])

    def test_that_when_listing_tags_succeeds_the_list_tags_method_returns_true(self):
        """
        tests True tags listed.
        """
        self.conn.describe_elasticsearch_domain.return_value = {
            "DomainStatus": domain_ret
        }
        result = boto_elasticsearch_domain.list_tags(
            DomainName=domain_ret["DomainName"], **conn_parameters
        )

        self.assertEqual(result["tags"], {})

    def test_that_when_listing_tags_fails_the_list_tags_method_returns_false(self):
        """
        tests False tags not listed.
        """
        self.conn.list_tags.side_effect = ClientError(error_content, "list_tags")
        self.conn.describe_elasticsearch_domain.return_value = {
            "DomainStatus": domain_ret
        }
        result = boto_elasticsearch_domain.list_tags(
            DomainName=domain_ret["DomainName"], **conn_parameters
        )
        self.assertTrue(result["error"])



"""
    :codeauthor: :email:`Shane Lee <slee@saltstack.com>`
"""

import textwrap

import salt.grains.disks as disks
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, mock_open, patch
from tests.support.unit import TestCase


class DisksGrainsTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for disks grains
    """

    def setup_loader_modules(self):
        return {
            disks: {"__salt__": {}},
        }

    def test__windows_disks(self):
        """
        Test grains._windows_disks, normal return
        Should return a populated dictionary
        """
        mock_which = MagicMock(return_value="C:\\Windows\\System32\\wbem\\WMIC.exe")
        wmic_result = textwrap.dedent(
            """
            DeviceId  MediaType
            0         4
            1         0
            2         3
            3         5
        """
        )
        mock_run_all = MagicMock(return_value={"stdout": wmic_result, "retcode": 0})

        with patch("salt.utils.path.which", mock_which), patch.dict(
            disks.__salt__, {"cmd.run_all": mock_run_all}
        ):
            result = disks._windows_disks()
            expected = {
                "ssds": ["\\\\.\\PhysicalDrive0"],
                "disks": [
                    "\\\\.\\PhysicalDrive0",
                    "\\\\.\\PhysicalDrive1",
                    "\\\\.\\PhysicalDrive2",
                    "\\\\.\\PhysicalDrive3",
                ],
            }
            self.assertDictEqual(result, expected)
            cmd = " ".join(
                [
                    "C:\\Windows\\System32\\wbem\\WMIC.exe",
                    "/namespace:\\\\root\\microsoft\\windows\\storage",
                    "path",
                    "MSFT_PhysicalDisk",
                    "get",
                    "DeviceID,MediaType",
                    "/format:table",
                ]
            )
            mock_run_all.assert_called_once_with(cmd)

    def test__windows_disks_retcode(self):
        """
        Test grains._windows_disks, retcode 1
        Should return empty lists
        """
        mock_which = MagicMock(return_value="C:\\Windows\\System32\\wbem\\WMIC.exe")
        mock_run_all = MagicMock(return_value={"stdout": "", "retcode": 1})
        with patch("salt.utils.path.which", mock_which), patch.dict(
            disks.__salt__, {"cmd.run_all": mock_run_all}
        ):
            result = disks._windows_disks()
            expected = {"ssds": [], "disks": []}
            self.assertDictEqual(result, expected)

    def test__linux_disks(self):
        """
        Test grains._linux_disks, normal return
        Should return a populated dictionary
        """

        files = [
            "/sys/block/asm!.asm_ctl_vbg0",
            "/sys/block/dm-0",
            "/sys/block/loop0",
            "/sys/block/ram0",
            "/sys/block/sda",
            "/sys/block/sdb",
            "/sys/block/vda",
        ]
        links = [
            "../devices/virtual/block/asm!.asm_ctl_vbg0",
            "../devices/virtual/block/dm-0",
            "../devices/virtual/block/loop0",
            "../devices/virtual/block/ram0",
            "../devices/pci0000:00/0000:00:1f.2/ata1/host0/target0:0:0/0:0:0:0/block/sda",
            "../devices/pci0000:35/0000:35:00.0/0000:36:00.0/host2/target2:1:0/2:1:0:0/block/sdb",
            "../devices/pci0000L00:0000:00:05.0/virtio2/block/vda",
        ]
        contents = [
            "1",
            "1",
            "1",
            "0",
            "1",
            "1",
            "1",
        ]

        patch_glob = patch("glob.glob", autospec=True, return_value=files)
        patch_readlink = patch(
            "salt.utils.path.readlink", autospec=True, side_effect=links
        )
        patch_fopen = patch("salt.utils.files.fopen", mock_open(read_data=contents))
        with patch_glob, patch_readlink, patch_fopen:
            ret = disks._linux_disks()

        assert ret == {"disks": ["sda", "sdb", "vda"], "ssds": []}, ret
